/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"



/* variables needed by 'compute_next_step()' */
static guint n;
static guint k;
static Matrix D = NULL;
static Matrix W = NULL;

/* a collection of matrix functions (see 'algorithm_initialize()') */
static MatrixFunctions matrixFunctions;


/* Computes the matriox product of E and F and stores the result in H. */
static void multiply(Matrix R, const Matrix E, const Matrix F);







static void compute(Matrix result, const Matrix initial)
{
    guint k, n;
    Matrix D;
    Semiring sr;


    /* some initializations */
    n = initial->size;
    sr = initial->semiring;

    /* create matrices needed in the computation */
    D = matrixFunctions.create_matrix();
    D->size = n;


    /* compute */
    for (k=0; k < n; k++)
    {
        sr->release(D->matrix[k][k]);
        D->matrix[k][k] = sr->one();
    }

    for (k=0; k < n-1; k++)
    {
        multiply(D, D, initial);
    }


    /* copy the result */
    matrixFunctions.copy_matrix(result, D);


    /* free unused matrix */
    matrixFunctions.free_matrix(D);
}

static void prepare(const Matrix initial)
{
    Semiring sr = initial->semiring;


    /* initialize all values which are needed by 'compute_next_step()' */

    n = initial->size;

    if (W != NULL) matrixFunctions.free_matrix(W);
    W = matrixFunctions.create_matrix();
    matrixFunctions.copy_matrix(W, initial);

    if (D != NULL) matrixFunctions.free_matrix(D);
    D = matrixFunctions.create_matrix();
    D->size = n;
    for (k=0; k < n; k++)
    {
        sr->release(D->matrix[k][k]);
        D->matrix[k][k] = sr->one();
    }

    k = 0;
}


static gboolean compute_next_step(Matrix result)
{
    /* compute */
    if (k < n-1)
    {
        multiply(D, D, W);

        k++;
    }


    if (D != NULL)
    {
        /* copy the result */
        matrixFunctions.copy_matrix(result, D);
    }


    /* returns TRUE if there are some steps left, otherwise FALSE */
    if (k < n-1)
    {
        return TRUE;
    }
    else
    {
        matrixFunctions.free_matrix(D);
        D = NULL;
        matrixFunctions.free_matrix(W);
        W = NULL;

        return FALSE;
    }
}



/* Computes the matriox product of E and F and stores the result in R. */
static void multiply(Matrix R, const Matrix E, const Matrix F)
{
    Semiring sr;
    Matrix H;
    guint u, v, t, n;


    /* some initializations */
    sr = E->semiring;
    n = E->size;
    H = matrixFunctions.create_matrix();
    H->size = n;


    /* computation */
    for (u=0; u < n; u++)
    {
        for (v=0; v < n; v++)
        {
            for (t=0; t < n; t++)
            {
                SemiringItem prod = sr->mult(E->matrix[u][t],
                                             F->matrix[t][v]);
                SemiringItem sum = sr->add(prod, H->matrix[u][v]);

                sr->release(H->matrix[u][v]);
                H->matrix[u][v] = sum;

                sr->release(prod);
            }
        }
    }


    /* copy the result */
    matrixFunctions.copy_matrix(R, H);


    /* free unused matrix */
    matrixFunctions.free_matrix(H);
}




G_MODULE_EXPORT void algorithm_initialize(Algorithm alg,
                                          const MatrixFunctions *mf)
{
    /* fill in the algorithm structure for the algorithm */
    alg->compute = compute;
    alg->prepare = prepare;
    alg->compute_next_step = compute_next_step;


    /* copy the matrix functions structure for our own use */
    matrixFunctions.create_matrix = mf->create_matrix;
    matrixFunctions.copy_matrix = mf->copy_matrix;
    matrixFunctions.free_matrix = mf->free_matrix;
}
