/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"



/* variables needed by 'compute_next_step()' */
static guint n;
static guint k;
static Matrix A = NULL;

/* a collection of matrix functions (see 'algorithm_initialize()') */
static MatrixFunctions matrixFunctions;




static void compute(Matrix result, const Matrix initial)
{
    guint k, i, j, n;
    Matrix A, H;
    Semiring sr;


    /* some initializations */
    n = initial->size;
    sr = initial->semiring;

    /* create matrices needed in the computation */
    A = matrixFunctions.create_matrix();
    matrixFunctions.copy_matrix(A, initial);

    H = matrixFunctions.create_matrix();
    H->size = n;


    /* compute */
    for (k=0; k < n; k++)
    {
        for (i=0; i < n; i++)
        {
            for (j=0; j < n; j++)
            {
                SemiringItem prod = sr->mult(A->matrix[i][k], A->matrix[k][j]);
                SemiringItem sum = sr->add(A->matrix[i][j], prod);

                sr->release(H->matrix[i][j]);
                H->matrix[i][j] = sum;

                sr->release(prod);
            }
        }
        matrixFunctions.copy_matrix(A, H);
    }


    /* copy the result */
    matrixFunctions.copy_matrix(result, A);


    /* free unused matrices */
    matrixFunctions.free_matrix(A);
    matrixFunctions.free_matrix(H);
}


static void prepare(const Matrix initial)
{
    /* initialize all values which are needed by 'compute_next_step()' */

    n = initial->size;

    if (A != NULL) matrixFunctions.free_matrix(A);
    A = matrixFunctions.create_matrix();
    matrixFunctions.copy_matrix(A, initial);

    k = 0;
}


static gboolean compute_next_step(Matrix result)
{
    guint i, j;
    Semiring sr = A->semiring;


    if (k < n)
    {
        /* initialize a matrix needed during computation */
        Matrix H = matrixFunctions.create_matrix();
        H->size = n;


        /* compute */
        for (i=0; i < n; i++)
        {
            for (j=0; j < n; j++)
            {
                SemiringItem prod = sr->mult(A->matrix[i][k], A->matrix[k][j]);
                SemiringItem sum = sr->add(A->matrix[i][j], prod);

                sr->release(H->matrix[i][j]);
                H->matrix[i][j] = sum;

                sr->release(prod);
            }
        }
        matrixFunctions.copy_matrix(A, H);
        matrixFunctions.free_matrix(H);
        k++;
    }


    if (A != NULL)
    {
        /* copy the result */
        matrixFunctions.copy_matrix(result, A);
    }


    /* returns TRUE if there are some steps left, otherwise FALSE */
    if (k < n)
    {
        return TRUE;
    }
    else
    {
        matrixFunctions.free_matrix(A);
        A = NULL;

        return FALSE;
    }
}


G_MODULE_EXPORT void algorithm_initialize(Algorithm alg,
                                          const MatrixFunctions *mf)
{
    /* fill in the algorithm structure for the algorithm */
    alg->compute = compute;
    alg->prepare = prepare;
    alg->compute_next_step = compute_next_step;


    /* copy the matrix functions structure for our own use */
    matrixFunctions.create_matrix = mf->create_matrix;
    matrixFunctions.copy_matrix = mf->copy_matrix;
    matrixFunctions.free_matrix = mf->free_matrix;
}
