/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* A list used throughout the model.
 * It contains objects of the same kind. One of these objects may "active".
 * There are also various events associated for which handler functions may be
 * registered.
 */

#ifndef MODEL_LIST_H
#define MODEL_LIST_H


#include "Model.h"



/* The opaque type of a model list. */
typedef struct ModelListStruct *ModelList;




/* Creates a new model list. */
ModelList m_list_new(void);


/* Frees the model list 'list'.
 * The stored items will not be freed. */
void m_list_free(ModelList list);



/* Returns the length of 'list'. */
guint m_list_length(ModelList list);

/* Returns the item at 'index' from 'list' or NULL if 'index' points beyond
 * the bounds of 'list'. */
gpointer m_list_get(ModelList list, guint index);



/* Adds 'data' to the end of 'list.
 *
 * The event 'OnItemAdded' is triggered. */
void m_list_add(ModelList list, gpointer data);

/* Removes the item at 'index' from 'list'.
 *
 * The events 'OnItemRemoved' and (maybe) 'OnActiveItemHasChanged' are
 * triggered. */
void m_list_remove(ModelList list, guint index);

/* Clears 'list'. After that 'list' contains no items anymore.
 *
 * The event 'OnListCleared' and the events of 'm_list_set_active_item()'
 * are triggered. */
void m_list_clear(ModelList list);



/* Calls 'function' with parameter 'data' on each item of 'list'. */
void m_list_foreach(ModelList list, GFunc function, gpointer data);



/* Sets the active item of 'list' to the one found at 'index'.
 * If no item should be active then pass 'G_MAXUINT' as 'index'.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active item. */
void m_list_set_active_item(ModelList list, guint index);

/* Returns the active item of 'list' or NULL if there is no active item. */
gpointer m_list_get_active_item(ModelList list);

/* Returns the active index of 'list' or G_MAXUINT if there is no active item.
 */
guint m_list_get_active_index(ModelList list);



/* Adds an event handler to 'list'. Every time 'event' is triggered
 * 'function' will be called. */
void m_list_add_hook(ModelList list, ModelEvent event, GHookFunc function);

/* Triggers 'event' for 'list'. All functions registered by 'm_list_add_hook()'
 * are called in their registration order. */
void m_list_trigger(ModelList list, ModelEvent event);



#endif /* MODEL_LIST_H */
