/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* Private functions of the model. These functions must not be called from any
 * other module. */

#ifndef MODEL_PRIVATE_H
#define MODEL_PRIVATE_H

#include "Model.h"
#include "Model.List.h"
#include "Model.Xml.h"


/* Initializes the algorithms module. */
void m_algorithms_initialize(void);

/* Cleans up the algorithms module. */
void m_algorithms_clean_up(void);



/* Initializes the semirings module. */
void m_semirings_initialize(void);

/* Cleans up the semirings module. */
void m_semirings_clean_up(void);



/* Initializes the matrices module. */
void m_matrices_initialize(void);

/* Cleans up the matrices module. */
void m_matrices_clean_up(void);



/* Initializes the computation module. */
void m_computation_initialize(void);

/* Cleans up the computation module. */
void m_computation_clean_up(void);





/* Creates a new matrix and loads its items from 'filename'.
 *
 * filename: Name of a file containing a matrix for the currently active
 *           semiring.
 *
 * Returns: NULL if an error occurred, otherwise a newly created matrix filled
 *          with the values found in 'filename'.
 *          Possible errors are:
 *             - There is no currently active semiring.
 *             - The file named 'filename' was not found.
 *             - The file does not contain a valid matrix.
 */
Matrix m_matrix_load(const gchar *filename);


/* Stores the matrix 'm' in a file named 'filename'. If this file already exists
 * it will be overwritten.
 *
 * m: The matrix which should be stored.
 *
 * filename: The name of the file where to store the matrix.
 */
void m_matrix_store(Matrix m, const gchar *filename);




/* Builds a list of filenames found in 'directory' having type 'type'.
 *
 * directory: The name of the directory in which to search for filenames.
 *
 * type: The ending of the files to find, e.g. ".c".
 *
 * returns: NULL on error. Otherwise a GSList holding the filenames
 *          of all found files or NULL.
 */
ModelList m_load_file_list(const gchar *directory, const gchar *type);



/* Extracts the filename without its ending from the given path.
 * For example, m_extract_name("some_path/file.ending") returns "file".
 *
 * filename: A path containing some filename.
 *
 * returns: A newly created string containing the name of the file without
 *          its ending found in 'filename'.
 *          The returned valued should be freed when it is not used anymore.
 */
gchar* m_extract_name(const gchar *filename);



/* Returns the path where to find the application and all dynamic modules, i.e.
 * algorithms, semirings and matrices. */
G_CONST_RETURN gchar* m_get_application_path(void);



#endif /* MODEL_PRIVATE_H */
