/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* A non-validating XML Parser. It scans (small) XML files and builds a
 * tree of nodes. It only supports a subset of XML, which includes
 * elements, attributes and text. */
#ifndef MODEL_XML_H
#define MODEL_XML_H


#include "Model.h"



/* The opaque type of an XML node. */
typedef struct XmlNodeStruct *XmlNode;


/* The possible types of an XML. */
typedef enum XmlNodeTypeEnum
{
    XmlElement,     /* the node is an XML element (with children) */
    XmlContent      /* the node contains only text */
} XmlNodeType;





/* Parse the file specified by 'filename' and build up a tree. This XML parser
 * is designed only for small files and only supports elements, attributes and
 * text.
 *
 * Returns: The root of the tree built up from the file 'filename'. */
XmlNode m_xml_parse(const gchar *filename);


/* Frees all the memory allocated for 'node' and its childs. */
void m_xml_free(XmlNode node);





/* Returns the type of 'node'. */
XmlNodeType m_xml_get_type(XmlNode node);


/* Returns the name of 'node' if the 'node' is an 'XmlElement', otherwise the
 * the text ('node' is of type 'XmlContent'). */
G_CONST_RETURN gchar* m_xml_get_name(XmlNode node);


/* Returns the attribute named with 'key' of 'node'. If there is no such
 * attribute then NULL is returned. */
G_CONST_RETURN gchar* m_xml_get_attribute(XmlNode node, const gchar *key);


/* Returns the parent node or NULL for the root node. */
XmlNode m_xml_get_parent(XmlNode node);



/* Returns the number of children of 'node'.  */
guint m_xml_get_number_of_children(XmlNode node);


/* Returns the child at position 'index' of 'node'. If there is no such child,
 * NULL is returned. */
XmlNode m_xml_get_child(XmlNode node, guint index);


/* Returns the next child with name 'name' of 'node', which follows 'prev'.
 * If there is no such child, NULL is returned.
 *
 * node: XML node for which a child should be found.
 *
 * name: Name of the child which should be found. May be NULL to return the node
 *       which follows 'prev'.
 *
 * prev: Pointer to the node which is a predecessor to the node which should be
 *       found. May be NULL to find the first node which matches 'name'.
 *
 * Returns: The first node which matches 'name' and comes after 'prev'.
 *          If there is no such child, NULL is returned. */
XmlNode m_xml_get_next_child(XmlNode node, const gchar *name, XmlNode prev);


/* Returns the content of 'node'.
 * If 'node' has more than one child only the the content of the first child
 * is returned.
 * If there is no first child of 'node' or it is not of type 'XmlContent'
 * NULL is returned. */
G_CONST_RETURN gchar* m_xml_get_content(XmlNode node);


#endif /* MODEL_XML_H */
