/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* List of all functions related to the model. */

#ifndef MODEL_H
#define MODEL_H


#include "AppC.h"
#include "Properties.h"



/* Initializes the model.
 *
 * applicationName: The filename and the path of this application as given
 *                  at the command line.
 */
void m_initialize(const gchar *applicationName);


/* Cleans up the model. */
void m_clean_up(void);





/* List of the possible events which may occur in the below functions.
 * See there for more information. */
typedef enum
{
    OnActiveItemWillChange,
    OnActiveItemHasChanged,
    OnItemAdded,
    OnItemRemoved,
    OnListCleared,

    OnSelectingPossibleChanged,

    OnComputationFinished,

    /* add new items above */
    NumberOfModelEvents
} ModelEvent;





/*****************************************************************************
 *
 * algorithm-specific functions
 *
 */


/* Returns the number of loaded algorithms. */
guint m_algorithms_length(void);


/* Returns the algorithm at 'index'.
 * If 'index' is greater than or equal to 'm_algorithms_length()' then NULL
 * is returned.
 * If the returned algorithm is the currently active one, then it is also
 * initialized, otherwise it should not be used for any computation. */
Algorithm m_algorithms_get(guint index);


/* Sets the active algorithm to the one found at 'index'.
 * If no algorithm should be active then pass 'G_MAXUINT' as 'index'.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_algorithms_select(guint index);


/* Returns the currently active algorithm or NULL if there is no algorithm
 * currently selected. */
Algorithm m_algorithms_get_selected(void);


/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_algorithms_add_hook(ModelEvent event, GHookFunc listener);






/*****************************************************************************
 *
 * semiring-specific functions
 *
 */


/* Returns TRUE if selecting semirings is possible, otherwise FALSE. */
gboolean m_semirings_is_selecting_possible(void);


/* Returns the number of loaded semirings suitable to the active algorithm. */
guint m_semirings_length(void);


/* Returns the semiring at 'index'.
 * If 'index' is greater than or equal to 'm_semirings_length()' then NULL is
 * returned. */
Semiring m_semirings_get(guint index);


/* Sets the active semiring to the one found at 'index'.
 * If no semiring should be active then pass 'G_MAXUINT' as 'index'.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_semirings_is_selecting_possible()').
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_semirings_select(guint index);


/* Returns the currently active semiring or NULL if there is no semiring
 * currently selected or selecting is not possible (see
 * 'm_semirings_is_selecting_possible()'). */
Semiring m_semirings_get_selected(void);


/* Returns the currently active index or G_MAXUINT if there is no semiring
 * currently selected or selecting is not possible (see
 * 'm_semirings_is_selecting_possible()'). */
guint m_semirings_get_selected_index(void);


/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_semirings_add_hook(ModelEvent event, GHookFunc listener);






/*****************************************************************************
 *
 * matrix-specific functions
 *
 */


/* Returns TRUE if selecting matrices is possible, otherwise FALSE. */
gboolean m_matrices_is_selecting_possible(void);


/* Returns the number of matrices currently available. */
guint m_matrices_length(void);


/* Returns the matrix at 'index'.
 * If 'index' is greater than or equal to 'm_matrices_length()' then NULL is
 * returned. */
Matrix m_matrices_get(guint index);


/* Sets the active matrix to the one found at 'index'.
 * If no matrix should be active then pass 'G_MAXUINT' as 'index'.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()').
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_matrices_select(guint index);


/* Returns the currently active matrix or NULL if there is no matrix currently
 * selected or selecting is not possible (see
 * 'm_matrices_is_selecting_possible()'). */
Matrix m_matrices_get_selected(void);


/* Returns the currently active index or G_MAXUINT if there is no matrix
 * currently selected or selecting is not possible (see
 * 'm_matrices_is_selecting_possible()'). */
guint m_matrices_get_selected_index(void);


/* Informs all listeners that some of the fields of the currently active matrix
 * have been changed.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered. if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_selected_modified(void);


/* Adds a new matrix the list of all current available matrices.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_add_new_matrix(void);


/* Removes the matrix at 'index' from the list of all current available
 * matrices.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_remove_matrix(guint index);


/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_matrices_add_hook(ModelEvent event, GHookFunc listener);





/* Creates a new matrix and fills its values with the current semiring's zero.
 * The new matrix has 'MAX_MATRIX_SIZE' rows and columns.
 * Its name is set to 'NULL'.
 *
 * Returns: NULL if there is no currently active semiring, otherwise a newly
 *          created matrix filled with the current semiring's zero element.
 */
Matrix m_matrix_new(void);


/* Frees the matrix 'm'. */
void m_matrix_free(Matrix m);


/* Copies the contents of 'source' to 'destination'. */
void m_matrix_copy(Matrix destination, const Matrix source);






/*****************************************************************************
 *
 * computation-specific functions
 *
 */


/* Defines the type of an executed computation. */
typedef enum
{
    NothingComputed,
    OnlyResultComputed,
    AllStepsComputed
} Computation;


/* Applies the currently selected algorithm to the initial matrix and computes
 * the result. If 'stepwise' is TRUE all intermediate matrices are stored, too.
 */
void m_computation_execute(gboolean stepwise);


/* Returns on of the above defined constants describing the last finished
 * computation. */
Computation m_computation_get_last(void);


/* Returns TRUE if there is a matrix following the second of the currently
 * active matrices. */
gboolean m_computation_has_next(void);


/* Returns TRUE if there is a matrix preceding the first of the currently
 * active matrices. */
gboolean m_computation_has_previous(void);


/* Actives the matrix following the currently active one if there is such a
 * matrix. */
void m_computation_select_next(void);


/* Actives the matrix preceding the currently active one if there is such a
 * matrix. */
void m_computation_select_previous(void);


/* Returns the index of the currently active matrix. */
guint m_computation_get_selected_index(void);


/* Returns the currently active matrix. */
Matrix m_computation_get_selected(void);


/* Returns the second of the currently active matrices. */
Matrix m_computation_get_2nd_selected(void);



/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_computation_add_hook(ModelEvent event, GHookFunc listener);



#endif /* MODEL_H */
