/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"



/* The underlying global data structure for this module.
 * It lists all found and loaded algorithms. */
static ModelList _algorithms;


/* A collection of matrix functions used in the algorithms. */
static MatrixFunctions _matrixFunctions;




/* Loads the algorithms information from 'root' and stores it in
 * 'algorithm'.
 * Returns: TRUE on success, FALSE otherwise. */
static gboolean _load_algorithm_info(Algorithm algorithm, XmlNode root)
{
    XmlNode node;
    const gchar *string;
    guint size, i;


    if (root == NULL) return FALSE;


    /* get the name of this algorithm */
    string = m_xml_get_attribute(root, "name");
    if (string != NULL) algorithm->name = g_strdup(string);
    else return FALSE;

    /* get the content of the node 'description' */
    node = m_xml_get_next_child(root, "description", NULL);
    if (node != NULL)
    {
        string = m_xml_get_content(node);
        if (string != NULL) algorithm->description = g_strdup(string);
        else return FALSE;
    }
    else return FALSE;

    /* build the list of all acceptable semirings */
    size = m_xml_get_number_of_children(root);
    algorithm->accepted_semirings = g_new0(gchar*, size + 1);
    i = 0;
    node = m_xml_get_next_child(root, "accepts", NULL);
    while (node != NULL)
    {
        string = m_xml_get_attribute(node, "semiring");
        if (string != NULL)
        {
            algorithm->accepted_semirings[i++] = g_strdup(string);
        }
        else return FALSE;

        node = m_xml_get_next_child(root, "accepts", node);
    }
    algorithm->accepted_semirings[i] = NULL;

    /* get the content of the node 'code' */
    node = m_xml_get_next_child(root, "code", NULL);
    if (node != NULL)
    {
        string = m_xml_get_content(node);
        if (string != NULL) algorithm->pseudocode = g_strdup(string);
        else return FALSE;
    }
    else return FALSE;


    return TRUE;
}


/* Free the algorithm passed as first argument. */
void _free_algorithm(gpointer item, gpointer notUsed)
{
    Algorithm algorithm = (Algorithm) item;
    guint i;

    if (algorithm != NULL)
    {
        g_free(algorithm->filename);
        g_free(algorithm->name);
        g_free(algorithm->description);
        g_free(algorithm->pseudocode);

        if (algorithm->accepted_semirings != NULL)
        {
            for (i=0; algorithm->accepted_semirings[i] != NULL; i++)
            {
                g_free(algorithm->accepted_semirings[i]);
            }
            g_free(algorithm->accepted_semirings);
        }

        if (algorithm->module != NULL) g_module_close(algorithm->module);

        g_free(algorithm);
    }
}


/* Loads an algorithm from the file specified by 'item'. */
static void _load_algorithm(gpointer item, gpointer notUsed)
{
    gchar *name = (gchar*) item;
    AlgorithmInitializer initialize;
    Algorithm algorithm;

    gchar *infofile;
    XmlNode root;


    algorithm = g_new0(struct AlgorithmStruct, 1);


    /* load the algorithm (dynamic module) */
    algorithm->module = g_module_open(name, G_MODULE_BIND_LAZY);
    if (algorithm->module == NULL)
    {
        _free_algorithm(algorithm, NULL);
        return;
    }

    /* get the algorithm's initializer function */
    if (!g_module_symbol(algorithm->module,
                         "algorithm_initialize",
                         (gpointer*) &initialize))
    {
        _free_algorithm(algorithm, NULL);
        return;
    }

    /* initialize the algorithm */
    initialize(algorithm, &_matrixFunctions);
    algorithm->filename = m_extract_name(name);


    /* load the info file and initialize the descriptive fields */
    infofile = g_strconcat(name, ".info", NULL);
    root = m_xml_parse(infofile);

    /* free some temporary data */
    g_free(infofile);
    g_free(name);

    /* load the content of the info file */
    if (!_load_algorithm_info(algorithm, root))
    {
        m_xml_free(root);
        _free_algorithm(algorithm, NULL);
        return;
    }

    /* free the info file tree */
    m_xml_free(root);


    /* add the loaded algorithm to the global list */
    m_list_add(_algorithms, algorithm);
}





/* Initializes the algorithms module. */
void m_algorithms_initialize(void)
{
    gchar* path;
    ModelList foundAlgorithms;

    _algorithms = m_list_new();


    /* initialize the matrix functions collection */
    _matrixFunctions.create_matrix = m_matrix_new;
    _matrixFunctions.copy_matrix = m_matrix_copy;
    _matrixFunctions.free_matrix = m_matrix_free;


    /* build the directory name where to find the algorithms */
    path = g_build_filename(m_get_application_path(), "algorithms", NULL);

    /* get a list of all algorithms found in the algorithms' directory */
    foundAlgorithms = m_load_file_list(path, ".algorithm");
    g_free(path);

    /* load all algorithms */
    m_list_foreach(foundAlgorithms, _load_algorithm, NULL);

    /* free unused list */
    m_list_free(foundAlgorithms);
}


/* Cleans up the algorithms module. */
void m_algorithms_clean_up(void)
{
    /* free all allocated algorithms */
    m_list_foreach(_algorithms, _free_algorithm, NULL);
    m_list_free(_algorithms);
}



/* Returns the number of loaded algorithms. */
guint m_algorithms_length(void)
{
    return m_list_length(_algorithms);
}


/* Returns the algorithm at 'index'.
 * If 'index' is greater than or equal to 'm_algorithms_length()' then NULL
 * is returned.
 * If the returned algorithm is the currently active one, then it is also
 * initialized, otherwise it should not be used for any computation. */
Algorithm m_algorithms_get(guint index)
{
    return (Algorithm) m_list_get(_algorithms, index);
}



/* Sets the active algorithm to the one found at 'index'.
 * If no algorithm should be active then pass 'G_MAXUINT' as 'index'.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_algorithms_select(guint index)
{
    m_list_set_active_item(_algorithms, index);
}

/* Returns the currently active algorithm or NULL if there is no algorithm
 * currently selected. */
Algorithm m_algorithms_get_selected(void)
{
    return (Algorithm) m_list_get_active_item(_algorithms);
}



/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_algorithms_add_hook(ModelEvent event, GHookFunc listener)
{
    m_list_add_hook(_algorithms, event, listener);
}
