/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"


/* the path where to find the application and all modules */
static gchar *_applicationDirectory;



/* Checks if 'string' ends with 'suffix'.
 *
 * returns: TRUE if 'string' ends with 'suffix, otherwise FALSE.
 */
static gboolean _has_suffix(const gchar *string, const gchar *suffix)
{
    guint stringLength = strlen(string);
    guint suffixLength = strlen(suffix);

    return (stringLength < suffixLength)
            ? FALSE
            : !strcmp(string + stringLength - suffixLength, suffix);
}





/* Initializes the model.
 *
 * applicationName: The filename and the path of this application as given
 *                  at the command line.
 */
void m_initialize(const gchar *applicationName)
{
    _applicationDirectory = g_path_get_dirname(applicationName);

    m_algorithms_initialize();
    m_semirings_initialize();
    m_matrices_initialize();
    m_computation_initialize();
}


/* Cleans up the model. */
void m_clean_up(void)
{
    m_computation_clean_up();
    m_matrices_clean_up();
    m_semirings_clean_up();
    m_algorithms_clean_up();

    g_free(_applicationDirectory);
}




/* Builds a list of filenames found in 'directory' having type 'type'.
 *
 * directory: The name of the directory in which to search for filenames.
 *
 * type: The ending of the files to find, e.g. ".c".
 *
 * returns: A list of all found filenames.
 */
ModelList m_load_file_list(const gchar* directory, const gchar* type)
{
    ModelList list;
    GDir *dir;
    const gchar *fn;


    /* create the list holding all found files */
    list = m_list_new();


    /* open the specified directory */
    dir = g_dir_open(directory, 0, NULL);
    if (dir == NULL)
    {
        return list;
    }

    /* collect all entries matching the given type */
    while ((fn = g_dir_read_name(dir)) != NULL)
    {
        gchar *filename = g_filename_to_utf8(fn, -1, NULL, NULL, NULL);
        if (filename == NULL) continue;

        /* if the current entry matches the type,
         * store the entry appended to its path */
        if (_has_suffix(filename, type))
        {
            gchar *completeFilename = g_build_filename(directory, filename,
                                                       NULL);
            m_list_add(list, completeFilename);
        }

        g_free(filename);
    }

    g_dir_close(dir);
    return list;
}



/* Extracts the filename without its ending from the given path.
 * For example, m_extract_name("some_path/file.ending") returns "file".
 *
 * filename: A path containing some filename.
 *
 * returns: A newly created string containing the name of the file without
 *          its ending found in 'filename'.
 *          The returned valued should be freed when it is not used anymore.
 */
gchar* m_extract_name(const gchar *filename)
{
    const gchar *begin, *end;

    begin = strrchr(filename, G_DIR_SEPARATOR);
    begin = (begin == NULL) ? filename : begin+1;

    end = strchr(begin+1, '.');
    end = (end == NULL) ? filename + strlen(filename) : end;

    return g_strndup(begin, end - begin);
}



/* Returns the path where to find the application and all dynamic modules, i.e.
 * algorithms, semirings and matrices. */
G_CONST_RETURN gchar* m_get_application_path(void)
{
    return _applicationDirectory;
}
