/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"


/* indicates whether the initial matrix has been set */
static gboolean _hasInitialMatrix;

/* indicates whether the last computation was done stepwise or not */
static Computation _lastComputation;



/* the list of all computed matrices */
static ModelList _computation;





/* Frees the matrix 'item' only when it is not equal to 'do_not_free'. */
static void _free_matrix(gpointer item, gpointer do_not_free)
{
    if (item != do_not_free)
    {
        m_matrix_free((Matrix) item);
    }
}




/* Called whenever the currently active initial matrix changed. */
static void _on_active_matrix_changed(gpointer notUsed)
{
    Matrix newMatrix = m_matrices_get_selected();

    /* clear list of computed matrices */
    m_list_foreach(_computation, _free_matrix, NULL);
    m_list_clear(_computation);

    /* add new initial matrix */
    _hasInitialMatrix = (newMatrix != NULL);
    if (_hasInitialMatrix)
    {
        Matrix copy = m_matrix_new();
        m_matrix_copy(copy, newMatrix);
        m_list_add(_computation, copy);
        m_list_set_active_item(_computation, 0);
    }
    _lastComputation = NothingComputed;
}




/* Initializes the computation module. */
void m_computation_initialize(void)
{
    _hasInitialMatrix = FALSE;
    _lastComputation = NothingComputed;

    _computation = m_list_new();

    m_matrices_add_hook(OnActiveItemHasChanged, _on_active_matrix_changed);

}


/* Cleans up the computation module. */
void m_computation_clean_up(void)
{
    m_list_foreach(_computation, _free_matrix, NULL);
    m_list_free(_computation);
}




/* Applies the currently selected algorithm to the initial matrix and computes
 * the result. If 'stepwise' is TRUE all intermediate matrices are stored, too.
 */
void m_computation_execute(gboolean stepwise)
{
    Algorithm algorithm = m_algorithms_get_selected();


    /* if the initial matrix has been set */
    if (_hasInitialMatrix)
    {
        Matrix initial = m_list_get(_computation, 0);

        if (!stepwise)  /* only compute the result */
        {
            if (_lastComputation != OnlyResultComputed)
            {
                Matrix result;

                /* clear the list without the initial matrix */
                m_list_foreach(_computation, _free_matrix, initial);
                m_list_clear(_computation);
                m_list_add(_computation, initial);
                m_list_set_active_item(_computation, 0);

                /* compute the result */
                result = m_matrix_new();
                algorithm->compute(result, initial);
                m_list_add(_computation, result);
                _lastComputation = OnlyResultComputed;

                /* inform all listeners */
                m_list_trigger(_computation, OnComputationFinished);
            }
        }
        else    /* compute all intermediate matrices and the result */
        {
            if (_lastComputation != AllStepsComputed)
            {
                Matrix result;
                gboolean again;

                /* clear the list without the initial matrix */
                m_list_foreach(_computation, _free_matrix, initial);
                m_list_clear(_computation);
                m_list_add(_computation, initial);
                m_list_set_active_item(_computation, 0);

                /* compute the result and all intermediate steps */
                algorithm->prepare(initial);
                again = TRUE;
                while (again)
                {
                    result = m_matrix_new();
                    again = algorithm->compute_next_step(result);
                    m_list_add(_computation, result);
                }
                _lastComputation = AllStepsComputed;

                /* inform all listeners */
                m_list_trigger(_computation, OnComputationFinished);
            }
        }
    }
}


/* Returns on of the above defined constants describing the last finished
 * computation. */
Computation m_computation_get_last(void)
{
    return _lastComputation;
}



/* Returns TRUE if there is a matrix following the second of the currently
 * active matrices. */
gboolean m_computation_has_next(void)
{
    guint selected = m_list_get_active_index(_computation);
    return (selected + 2 < m_list_length(_computation));
}


/* Returns TRUE if there is a matrix preceding the first of the currently
 * active matrices. */
gboolean m_computation_has_previous(void)
{
    return (m_list_get_active_index(_computation) > 0);
}


/* Actives the matrix following the currently active one if there is such a
 * matrix. */
void m_computation_select_next(void)
{
    if (m_computation_has_next())
    {
        guint selected = m_list_get_active_index(_computation);
        m_list_set_active_item(_computation, selected + 1);
    }
}


/* Actives the matrix preceding the currently active one if there is such a
 * matrix. */
void m_computation_select_previous(void)
{
    if (m_computation_has_previous())
    {
        guint selected = m_list_get_active_index(_computation);
        m_list_set_active_item(_computation, selected - 1);
    }
}


/* Returns the index of the currently active matrix. */
guint m_computation_get_selected_index(void)
{
    return m_list_get_active_index(_computation);
}


/* Returns the currently active matrix. */
Matrix m_computation_get_selected(void)
{
    return (Matrix) m_list_get_active_item(_computation);
}


/* Returns the second of the currently active matrices. */
Matrix m_computation_get_2nd_selected(void)
{
    guint selected = m_list_get_active_index(_computation);
    return (Matrix) m_list_get(_computation, selected + 1);
}




/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_computation_add_hook(ModelEvent event, GHookFunc listener)
{
    m_list_add_hook(_computation, event, listener);
}
