/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"


/* The internal structure of a model list. */
struct ModelListStruct
{
    /* Holds the items stored in the model list. */
    GPtrArray *array;

    /* The currently active index in 'array'. */
    guint activeIndex;


    /* List of the registered event handlers. */
    GHookList hooks[NumberOfModelEvents];
};





/* Creates a new model list. */
ModelList m_list_new(void)
{
    ModelList list;
    guint i;

    list = g_new0(struct ModelListStruct, 1);

    list->array = g_ptr_array_new();
    list->activeIndex = G_MAXUINT;

    for (i=0; i < NumberOfModelEvents; i++)
    {
        g_hook_list_init(&(list->hooks[i]), sizeof(GHook));
    }

    return list;
}


/* Frees the model list 'list'. */
void m_list_free(ModelList list)
{
    guint i;

    g_ptr_array_free(list->array, FALSE);

    for (i=0; i < NumberOfModelEvents; i++)
    {
        g_hook_list_clear(&(list->hooks[i]));
    }

}



/* Returns the length of 'list'. */
guint m_list_length(ModelList list)
{
    return list->array->len;
}


/* Returns the item at 'index' from 'list' or NULL if 'index' points beyond
 * the bounds of 'list'. */
gpointer m_list_get(ModelList list, guint index)
{
    return (index < m_list_length(list))
            ? list->array->pdata[index]
            : NULL;
}



/* Adds 'data' to the end of 'list.
 *
 * The event 'OnItemAdded' is triggered. */
void m_list_add(ModelList list, gpointer data)
{
    g_ptr_array_add(list->array, data);

    m_list_trigger(list, OnItemAdded);
}


/* Removes the item at 'index' from 'list'.
 *
 * The events 'OnItemRemoved' and (maybe) 'OnActiveItemHasChanged' are
 * triggered. */
void m_list_remove(ModelList list, guint index)
{
    if (index < m_list_length(list))
    {
        g_ptr_array_remove_index(list->array, index);

        m_list_trigger(list, OnItemRemoved);

        /* if the active item was just removed */
        if (list->activeIndex == index)
        {
            /* there is no active item anymore */
            m_list_set_active_item(list, G_MAXUINT);
        }
    }
}


/* Clears 'list'. After that 'list' contains no items anymore.
 *
 * The events 'OnListCleared' and 'OnActiveItemHasChanged' are triggered. */
void m_list_clear(ModelList list)
{
    if (m_list_length(list) > 0)
    {
        gint i;

        for (i=m_list_length(list)-1; i >= 0; i--)
        {
            g_ptr_array_remove_index(list->array, i);
        }

        m_list_trigger(list, OnListCleared);

        /* there is no active item anymore */
        m_list_set_active_item(list, G_MAXUINT);
    }
}



/* Calls 'function' with parameter 'data' on each item of 'list'. */
void m_list_foreach(ModelList list, GFunc function, gpointer data)
{
    guint index;

    for (index=0; index < m_list_length(list); index++)
    {
        function(m_list_get(list, index), data);
    }
}



/* Sets the active item of 'list' to the one found at 'index'.
 * If no item should be active then pass 'G_MAXUINT' as 'index'.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active item. */
void m_list_set_active_item(ModelList list, guint index)
{
    if (index != list->activeIndex)
    {
        m_list_trigger(list, OnActiveItemWillChange);
        list->activeIndex = index;
        m_list_trigger(list, OnActiveItemHasChanged);
    }
}


/* Returns the active item of 'list' or NULL if there is no active item. */
gpointer m_list_get_active_item(ModelList list)
{
    return m_list_get(list, list->activeIndex);
}


/* Returns the active index of 'list' or G_MAXUINT if there is no active item.
 */
guint m_list_get_active_index(ModelList list)
{
    return list->activeIndex;
}




/* Adds an event handler to 'list'. Every time 'event' is triggered
 * 'function' will be called. */
void m_list_add_hook(ModelList list, ModelEvent event, GHookFunc function)
{
    GHook *hook = g_hook_alloc(&(list->hooks[event]));

    hook->func = function;
    hook->data = NULL;

    g_hook_append(&(list->hooks[event]), hook);
}


/* Triggers 'event' for 'list'. All functions registered by 'm_list_add_hook()'
 * are called in their registration order. */
void m_list_trigger(ModelList list, ModelEvent event)
{
    g_hook_list_invoke(&(list->hooks[event]), FALSE);
}
