/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"
#include "Properties.h"




/* A list of all currently loaded matrices. */
static ModelList _matrices;


/* Indicates whether a semiring can be selected or not.
 * This variable should only be changed by '_set_selecting_possible()'. */
static gboolean _isSelectingPossible;





/* Sets if selecting a semiring is possible or not.
 *
 * The event 'OnSelectingPossibleChanged' is triggered if the state is changed.
 */
static void _set_selecting_possible(gboolean newState)
{
    if (_isSelectingPossible != newState)
    {
        _isSelectingPossible = newState;
        m_list_trigger(_matrices, OnSelectingPossibleChanged);
    }
}


/* Builds the filename of the matrix 'm'. */
static gchar* _build_matrix_filename(Matrix m)
{
    gchar *name = g_strconcat(m->name, ".", m->semiring->filename, ".matrix",
                              NULL);
    gchar *filename = g_build_filename(prop_get_matrices_directory(),
                                       name,
                                       NULL);

    g_free(name);

    return filename;
}


/* Loads a matrix from the file specified by 'item'. */
static void _load_matrix(gpointer item, gpointer notUsed)
{
    Matrix matrix = m_matrix_load((gchar*) item);
    if (matrix != NULL) m_list_add(_matrices, matrix);
    g_free(item);
}


/* Stores the matrix 'item'. */
static void _store_matrix(gpointer item, gpointer notUsed)
{
    Matrix m = (Matrix) item;

    /* build filename */
    gchar *filename = _build_matrix_filename(m);

    /* store the matrix */
    m_matrix_store(m, filename);

    g_free(filename);
}


/* Frees the matrix 'item'. */
static void _free_matrix(gpointer item, gpointer notUsed)
{
    m_matrix_free((Matrix) item);
}



/* Called whenever the event 'OnSelectingPossibleChanged' occurs in the
 * semirings module. */
static void _on_semirings_selecting_possible_changed(gpointer notUsed)
{
    if (!m_semirings_is_selecting_possible())
    {
        /* if selecting semiring is disabled, then disable selecting matrices,
         * too */
        _set_selecting_possible(FALSE);
    }
}



/* Called whenever the event 'OnActiveItemWillChange' occurs in the
 * semirings module. */
static void _on_active_semiring_will_change(gpointer notUsed)
{
    /* save and free all matrices */
    m_list_foreach(_matrices, _store_matrix, NULL);
    m_list_foreach(_matrices, _free_matrix, NULL);

    m_list_clear(_matrices);
}



/* Called whenever the event 'OnActiveItemHasChanged' occurs in the
 * semirings module. */
static void _on_active_semiring_has_changed(gpointer notUsed)
{
    Semiring semiring = m_semirings_get_selected();

    /* load matrices (if a semiring is currently selected) */
    if (semiring != NULL)
    {
        ModelList foundMatrices;
        gchar *pattern = g_strconcat(".",
                                     semiring->filename,
                                     ".matrix",
                                     NULL);

        /* find all matrices appropriate for the active semiring */
        foundMatrices = m_load_file_list(prop_get_matrices_directory(),
                                         pattern);

        /* load all found matrices */
        m_list_foreach(foundMatrices, _load_matrix, NULL);

        /* free unused list */
        m_list_free(foundMatrices);

        /* allow selecting one of the loaded matrices */
        _set_selecting_possible(TRUE);
    }
    else /* there is no semiring selected */
    {
        /* then forbid selecting a matrix */
        _set_selecting_possible(FALSE);
    }
}





/* Initializes the matrices module. */
void m_matrices_initialize(void)
{
    _isSelectingPossible = FALSE;
    _matrices = m_list_new();


    m_semirings_add_hook(OnSelectingPossibleChanged,
                         _on_semirings_selecting_possible_changed);

    m_semirings_add_hook(OnActiveItemWillChange,
                         _on_active_semiring_will_change);

    m_semirings_add_hook(OnActiveItemHasChanged,
                         _on_active_semiring_has_changed);
}


/* Cleans up the matrices module. */
void m_matrices_clean_up(void)
{
    m_list_foreach(_matrices, _store_matrix, NULL);
    m_list_foreach(_matrices, _free_matrix, NULL);
    m_list_free(_matrices);
}





/* Returns TRUE if selecting matrices is possible, otherwise FALSE. */
gboolean m_matrices_is_selecting_possible(void)
{
    return _isSelectingPossible;
}



/* Returns the number of matrices currently available. */
guint m_matrices_length(void)
{
    return m_list_length(_matrices);
}


/* Returns the matrix at 'index'.
 * If 'index' is greater than or equal to 'm_matrices_length()' then NULL is
 * returned. */
Matrix m_matrices_get(guint index)
{
    return (Matrix) m_list_get(_matrices, index);
}



/* Sets the active matrix to the one found at 'index'.
 * If no matrix should be active then pass 'G_MAXUINT' as 'index'.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()').
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_matrices_select(guint index)
{
    if (m_matrices_is_selecting_possible())
    {
        m_list_set_active_item(_matrices, index);
    }
}


/* Returns the currently active matrix or NULL if there is no matrix currently
 * selected or selecting is not possible (see
 * 'm_matrices_is_selecting_possible()'). */
Matrix m_matrices_get_selected(void)
{
    return (m_matrices_is_selecting_possible())
            ? (Matrix) m_list_get_active_item(_matrices)
            : NULL;
}



/* Returns the currently active index or G_MAXUINT if there is no matrix
 * currently selected or selecting is not possible (see
 * 'm_matrices_is_selecting_possible()'). */
guint m_matrices_get_selected_index(void)
{
    return (m_matrices_is_selecting_possible())
            ? m_list_get_active_index(_matrices)
            : G_MAXUINT;
}



/* Informs all listeners that some of the fields of the currently active matrix
 * have been changed.
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered. if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_selected_modified(void)
{
    if (m_matrices_is_selecting_possible())
    {
        m_list_trigger(_matrices, OnActiveItemWillChange);
        m_list_trigger(_matrices, OnActiveItemHasChanged);
    }
}




/* Adds a new matrix the list of all current available matrices.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_add_new_matrix(void)
{
    if (m_matrices_is_selecting_possible())
    {
        Matrix matrix = m_matrix_new();
        if (matrix != NULL) m_list_add(_matrices, matrix);
    }
}



/* Removes the matrix at 'index' from the list of all current available
 * matrices.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_matrices_is_selecting_possible()'). */
void m_matrices_remove_matrix(guint index)
{
    if (m_matrices_is_selecting_possible())
    {
        Matrix matrix = m_matrices_get(index);

        if (matrix != NULL)
        {
            /* build the filename and remove the matrix file */
            gchar *filename = _build_matrix_filename(matrix);
            remove(filename);
            g_free(filename);

            /* remove the matrix from the list */
            m_matrix_free(matrix);
            m_list_remove(_matrices, index);
        }
    }
}



/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_matrices_add_hook(ModelEvent event, GHookFunc listener)
{
    m_list_add_hook(_matrices, event, listener);
}
