/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"



/* Creates a new matrix and fills its values with the current semiring's zero.
 * The new matrix has 'MAX_MATRIX_SIZE' rows and columns.
 * Its name is set to 'NULL'.
 *
 * Returns: NULL if there is no currently active semiring, otherwise a newly
 *          created matrix filled with the current semiring's zero element.
 */
Matrix m_matrix_new(void)
{
    Semiring sr = m_semirings_get_selected();

    if (sr == NULL)
    {
        return NULL;
    }
    else
    {
        guint row, column;
        Matrix m = g_new0(struct MatrixStruct, 1);

        /* fill the matrix with the semiring's zero element */
        m->matrix = g_new0(SemiringItem*, prop_get_max_matrix_size());
        for (row=0; row < prop_get_max_matrix_size(); row++)
        {
            m->matrix[row] = g_new0(SemiringItem, prop_get_max_matrix_size());
            for (column=0; column < prop_get_max_matrix_size(); column++)
            {
                m->matrix[row][column] = sr->zero();
            }
        }

        m->size = prop_get_max_matrix_size();
        m->name = NULL;
        m->semiring = sr;

        return m;
    }
}



/* Frees the matrix 'm'. */
void m_matrix_free(Matrix m)
{
    if (m != NULL)
    {
        guint row, column;
        Semiring sr = m->semiring;

        g_free(m->name);

        for (row=0; row < prop_get_max_matrix_size(); row++)
        {
            for (column=0; column < prop_get_max_matrix_size(); column++)
            {
                sr->release(m->matrix[row][column]);
            }
            g_free(m->matrix[row]);
        }
        g_free(m->matrix);
    }
}



/* Copies the contents of 'source' to 'destination'. */
void m_matrix_copy(Matrix destination, const Matrix source)
{
    guint row, column;
    Semiring sr;

    /* copy name, semiring and size */
    destination->name = g_strdup(source->name);
    destination->semiring = source->semiring;
    destination->size = source->size;

    /* copy the matrix itself */
    sr = destination->semiring;
    for (row=0; row < destination->size; row++)
    {
        for (column=0; column < destination->size; column++)
        {
            Semiring copy = sr->copy(source->matrix[row][column]);
            sr->release(destination->matrix[row][column]);
            destination->matrix[row][column] = copy;

        }
    }
}



/* Creates a new matrix and loads its items from 'filename'.
 *
 * filename: Name of a file containing a matrix for the currently active
 *           semiring.
 *
 * Returns: NULL if an error occurred, otherwise a newly created matrix filled
 *          with the values found in 'filename'.
 *          Possible errors are:
 *             - There is no currently active semiring.
 *             - The file named 'filename' was not found.
 *             - The file does not contain a valid matrix.
 *             - The matrix size in the file was larger than then the global
 *               matrix size (as returned by prop_get_max_matrix_size()).
 */
Matrix m_matrix_load(const gchar *filename)
{
    XmlNode root, xRow, xItem;
    Matrix m;
    guint row, column, size;
    gchar *p;
    const gchar *xSize;

    /* parse the file and create a tree */
    root = m_xml_parse(filename);
    if (root == NULL)
    {
        return NULL;
    }

    /* get the size of the matrix */
    xSize = m_xml_get_attribute(root, "size");
    if (xSize == NULL)
    {
        return NULL;
    }
    size = strtoul(xSize, &p, 10);
    if (p == xSize || size == ULONG_MAX || size > prop_get_max_matrix_size())
    {
        return NULL;
    }

    /* create and initialize a new matrix */
    m = m_matrix_new();
    m->name = m_extract_name(filename);
    m->size = size;

    /* fill the matrix with the values found in the file */
    xRow = NULL;
    for (row=0; row<size; row++)
    {
        /* find the next 'row' node */
        xRow = m_xml_get_next_child(root, "row", xRow);
        if (xRow == NULL)
        {
            return NULL;
        }

        xItem = NULL;
        for (column=0; column<size; column++)
        {
            XmlNode xItemContent;

            /* find the next 'item' node */
            xItem = m_xml_get_next_child(xRow, "item", xItem);
            if (xItem == NULL ||
                (xItemContent = m_xml_get_child(xItem, 0)) == NULL)
            {
                return NULL;
            }
            else
            {
                /* fill in the value of this item */
                const gchar* value = m_xml_get_name(xItemContent);
                m->semiring->release(m->matrix[row][column]);
                m->matrix[row][column] = m->semiring->from_string(value, NULL);
            }
        }
    }
    return m;
}



/* Stores the matrix 'm' in a file named 'filename'. If this file already exists
 * it will be overwritten.
 *
 * m: The matrix which should be stored.
 *
 * filename: The name of the file where to store the matrix.
 */
void m_matrix_store(Matrix m, const gchar *filename)
{
    FILE *out;
    guint row, column;
    gchar *string;


    /* open the file (possibly overwrite the existing file) */
    out = fopen(filename, "w");
    if (out == NULL) return;


    fputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", out);
    fputs("<!DOCTYPE matrix SYSTEM \"matrix.dtd\">\n", out);

    fprintf(out, "\n<matrix size=\"%u\">\n", m->size);


    /* write the matrix */
    for (row=0; row < m->size; row++)
    {
        fputs("<row>\n", out);

        for (column=0; column < m->size; column++)
        {
            fputs("<item>", out);

            string = m->semiring->to_string(m->matrix[row][column]);
            fputs(string, out);
            g_free(string);

            fputs("</item>\n", out);
        }
        fputs("</row>\n", out);
    }

    fputs("</matrix>\n", out);
    fclose(out);
}
