/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"




/* Collects all found and loaded semirings. */
static ModelList _allSemirings;

/* Lists all semirings suitable to the current algorithm. This list is a subset
 * of '_allSemirings'. */
static ModelList _suitableSemirings;


/* Indicates whether a semiring can be selected or not.
 * This variable should only be changed by '_set_selecting_possible()'. */
static gboolean _isSelectingPossible;





/* Sets if selecting a semiring is possible or not.
 *
 * The event 'OnSelectingPossibleChanged' is triggered if the state is changed.
 */
static void _set_selecting_possible(gboolean newState)
{
    if (_isSelectingPossible != newState)
    {
        _isSelectingPossible = newState;
        m_list_trigger(_suitableSemirings, OnSelectingPossibleChanged);
    }
}


/* Loads the semiring information from 'root' and stores it in
 * 'semiring'.
 * Returns: TRUE on success, FALSE otherwise. */
static gboolean _load_semiring_info(Semiring semiring, XmlNode root)
{
    XmlNode node;
    const gchar *string;


    if (root == NULL) return FALSE;


    /* get the name of the semiring */
    string = m_xml_get_attribute(root, "name");
    if (string != NULL) semiring->name = g_strdup(string);
    else return FALSE;

    /* get the content of the node 'description' */
    node = m_xml_get_next_child(root, "description", NULL);
    if (node != NULL)
    {
        string = m_xml_get_content(node);
        if (string != NULL) semiring->description = g_strdup(string);
        else return FALSE;
    }
    else return FALSE;


    return TRUE;
}


/* Free the semiring passed as first argument. */
void _free_semiring(gpointer item, gpointer notUsed)
{
    Semiring semiring = (Semiring) item;

    if (semiring != NULL)
    {
        g_free(semiring->filename);
        g_free(semiring->name);
        g_free(semiring->description);

        if (semiring->module != NULL) g_module_close(semiring->module);

        g_free(semiring);
    }
}


/* Loads a semiring from the file specified by 'item'. */
static void _load_semiring(gpointer item, gpointer notUsed)
{
    gchar *name = (gchar*) item;
    SemiringInitializer initialize;
    Semiring semiring;

    gchar *infofile;
    XmlNode root;


    semiring = g_new0(struct SemiringStruct, 1);


    /* load the semiring (dynamic module) */
    semiring->module = g_module_open(name, G_MODULE_BIND_LAZY);
    if (semiring->module == NULL) return;

    /* get the semiring's initializer function */
    if (!g_module_symbol(semiring->module,
                         "semiring_initialize",
                         (gpointer*) &initialize))
    {
        _free_semiring(semiring, NULL);
        return;
    }

    /* initialize the semiring */
    initialize(semiring);
    semiring->filename = m_extract_name(name);

    /* load the info file and initialize the descriptive fields */
    infofile = g_strconcat(name, ".info", NULL);
    root = m_xml_parse(infofile);

    /* free some temporary data */
    g_free(infofile);
    g_free(name);


    if (!_load_semiring_info(semiring, root))
    {
        _free_semiring(semiring, NULL);
        m_xml_free(root);
        return;
    }


    /* free the info file tree */
    m_xml_free(root);


    /* add the loaded semiring to the list of all semirings */
    m_list_add(_allSemirings, semiring);
}



/* Adds the semiring 'item' to '_suitableSemirings' if the algorithm 'data'
 * accepts this semiring. */
static void _filter_semiring(gpointer item, gpointer data)
{
    Semiring semiring = (Semiring) item;
    Algorithm algorithm = (Algorithm) data;

    /* check if 'semiring' is acceptable by 'algorithm' */
    if (algorithm->accepted_semirings != NULL &&
        algorithm->accepted_semirings[0] != NULL)
    {
        char **p = algorithm->accepted_semirings;
        while (*p != NULL && strcmp(*p, semiring->filename)) p++;
        if (*p == NULL) return;
    }

    m_list_add(_suitableSemirings, semiring);
}



/* Listener function called whenever the active algorithm changes. */
static void _on_active_algorithm_has_changed(gpointer notUsed)
{
    Algorithm algorithm = m_algorithms_get_selected();


    /* clear the '_suitableSemirings' list (after that, there is no active
     * semiring anymore) */
    m_list_clear(_suitableSemirings);

    /* if there is a new active algorithm: */
    if (algorithm != NULL)
    {
        /* build up a list of all suitable semirings */
        m_list_foreach(_allSemirings, _filter_semiring, algorithm);

        /* selecting a semiring is now possible */
        _set_selecting_possible(TRUE);
    }
    else /* there is no active algorithm: */
    {
        /* selecting a semiring is not possible in this case */
        _set_selecting_possible(FALSE);
    }
}






/* Initializes the semirings module. */
void m_semirings_initialize(void)
{
    gchar* path;
    ModelList foundSemirings;

    _isSelectingPossible = FALSE;
    _allSemirings = m_list_new();
    _suitableSemirings = m_list_new();

    /* build the directory name where to find the algorithms */
    path = g_build_filename(m_get_application_path(), "semirings", NULL);

    /* get a list of all semirings found in the semirings' directory */
    foundSemirings = m_load_file_list(path, ".semiring");
    g_free(path);


    /* load all semirings */
    m_list_foreach(foundSemirings, _load_semiring, NULL);

    /* free unused list */
    m_list_free(foundSemirings);

    /* add listener to be informed whenever the active algorithm is changed */
    m_algorithms_add_hook(OnActiveItemHasChanged,
                          _on_active_algorithm_has_changed);
}


/* Cleans up the semirings module. */
void m_semirings_clean_up(void)
{
    /* free all allocated semirings */
    m_list_foreach(_allSemirings, _free_semiring, NULL);
    m_list_free(_allSemirings);
}



/* Returns TRUE if selecting semirings is possible, otherwise FALSE. */
gboolean m_semirings_is_selecting_possible(void)
{
    return _isSelectingPossible;
}



/* Returns the number of loaded semirings suitable to the active algorithm. */
guint m_semirings_length(void)
{
    return m_list_length(_suitableSemirings);
}


/* Returns the semiring at 'index'.
 * If 'index' is greater than or equal to 'm_semirings_length()' then NULL is
 * returned. */
Semiring m_semirings_get(guint index)
{
    return (Semiring) m_list_get(_suitableSemirings, index);
}



/* Sets the active semiring to the one found at 'index'.
 * If no semiring should be active then pass 'G_MAXUINT' as 'index'.
 *
 * Notice that this function is only successful if selecting is possible
 * (see 'm_semirings_is_selecting_possible()').
 *
 * The events 'OnActiveItemWillChange' and 'OnActiveItemHasChanged' are
 * triggered if 'index' does not point to the currently active but some other
 * valid item. */
void m_semirings_select(guint index)
{
    if (m_semirings_is_selecting_possible())
    {
        m_list_set_active_item(_suitableSemirings, index);
    }
}

/* Returns the currently active semiring or NULL if there is no semiring
 * currently selected or selecting is not possible (see
 * 'm_semirings_is_selecting_possible()'). */
Semiring m_semirings_get_selected(void)
{
    return (m_semirings_is_selecting_possible())
            ? (Semiring) m_list_get_active_item(_suitableSemirings)
            : NULL;
}

/* Returns the currently active index or G_MAXUINT if there is no semiring
 * currently selected or selecting is not possible (see
 * 'm_semirings_is_selecting_possible()'). */
guint m_semirings_get_selected_index(void)
{
    return (m_semirings_is_selecting_possible())
            ? m_list_get_active_index(_suitableSemirings)
            : G_MAXUINT;
}



/* Adds an event handler. Every time 'event' is triggered, 'listener' will be
 * called. */
void m_semirings_add_hook(ModelEvent event, GHookFunc listener)
{
    m_list_add_hook(_suitableSemirings, event, listener);
}
