/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"


/* The structure of an XmlNode. */
struct XmlNodeStruct
{
    /* Type of this node. */
    XmlNodeType type;

    /* The name of the tag (for XmlElement) or the text (for XmlContent). */
    gchar *name;

    /* List of the attributes (keys followed by their values). */
    ModelList attributes;

    /* The parent node. NULL for the root node. */
    XmlNode parent;

    /* The list of this node's children. */
    ModelList children;
};





/* Called by the parser whenever a start element was found.
 * Creates a new node and adds it to the node stored in 'user_data',
 * which is then replaced by the new node. */
static void _start(GMarkupParseContext *context,
                   const gchar         *element_name,
                   const gchar        **attribute_names,
                   const gchar        **attribute_values,
                   gpointer             user_data,
                   GError             **error)
{
    XmlNode *parent = (XmlNode *) user_data;
    guint index;

    /* create a new node and fill up its fields */
    XmlNode child = g_new0(struct XmlNodeStruct, 1);
    child->name = g_strdup(element_name);
    child->type = XmlElement;
    child->parent = *parent;
    child->children = m_list_new();

    /* store the attributes  */
    child->attributes = m_list_new();

    for (index=0;
         attribute_names[index] != NULL && attribute_values[index] != NULL;
         index++)
    {
        m_list_add(child->attributes, g_strdup(attribute_names[index]));
        m_list_add(child->attributes, g_strdup(attribute_values[index]));
    }

    if (*parent != NULL)
    {
        m_list_add((*parent)->children, child);
    }

    *parent = child;
}


/* Called by the parser whenever an end element was found.
 * Replaces the node found in 'user_data' by its parent. */
static void _end(GMarkupParseContext *context,
                 const gchar         *element_name,
                 gpointer             user_data,
                 GError             **error)
{
    XmlNode *node = (XmlNode *) user_data;

    if ((*node)->parent != NULL)
    {
        *node = (*node)->parent;
    }
}


/* Called by the parser whenever a text element was found.
 * Removes leading whitespace and creates a new node which is added to the node
 * found ind 'user_data'. */
static void _text(GMarkupParseContext *context,
                  const gchar         *text,
                  gsize                text_len,
                  gpointer             user_data,
                  GError             **error)
{
    XmlNode *node = (XmlNode *) user_data;

    const gchar *p = text;

    /* scan leading whitespace */
    while (p - text < text_len && g_ascii_isspace(*p)) p++;

    /* if there is really some text, create a new node */
    if (p - text < text_len)
    {
        XmlNode child = g_new0(struct XmlNodeStruct, 1);
        child->name = g_strndup(p, text_len - (p - text));
        child->type = XmlContent;
        child->attributes = NULL;
        child->parent = *node;

        m_list_add((*node)->children, child);
    }
}



/* Parse the file specified by 'filename' and build up a tree. This XML parser
 * is designed only for small files and only supports elements and text, no
 * attributes.
 *
 * Returns: The root of the tree built up from the file 'filename'. */
XmlNode m_xml_parse(const gchar *filename)
{
    GMarkupParser parser;
    GMarkupParseContext *context;

    GIOChannel *file;
    GIOStatus status;

    gchar *data;
    gsize length;

    XmlNode root = NULL;


    /* open the file */
    file = g_io_channel_new_file(filename, "r", NULL);
    if (file == NULL)
    {
        return NULL;
    }

    /* read the complete file content */
    status = g_io_channel_read_to_end(file, &data, &length, NULL);
    if (status == G_IO_STATUS_NORMAL)
    {
        /* configure the parser */
        parser.start_element = _start;
        parser.end_element = _end;
        parser.text = _text;
        parser.passthrough = NULL;
        parser.error = NULL;

        /* initialize the parser */
        context = g_markup_parse_context_new(&parser, 0, &root, NULL);

        /* parse the file using the above functions */
        if (!g_markup_parse_context_parse(context, data, length, NULL))
        {
            g_free(data);
            g_io_channel_close(file);
            return NULL;
        }
    }

    g_free(data);
    g_io_channel_close(file);

    return root;
}




/* Frees all the memory allocated for 'node' and its childs. */
void m_xml_free(XmlNode node)
{
    if (node != NULL)
    {
        guint i;

        g_free(node->name);

        /* free all attributes and their values */
        if (node->attributes != NULL)
        {
            for (i=0; i < m_list_length(node->attributes); i++)
            {
                g_free(m_list_get(node->attributes, i));
            }
            m_list_free(node->attributes);
        }

        /* free all children recursively */
        if (node->children != NULL)
        {
            for (i=0; i < m_list_length(node->children); i++)
            {
                m_xml_free((XmlNode) m_list_get(node->children, i));
            }
            m_list_free(node->children);
        }

        g_free(node);
    }
}




/* Returns the type of 'node'. */
XmlNodeType m_xml_get_type(XmlNode node)
{
    return node->type;
}

/* Returns the name of 'node' if the 'node' is an 'XmlElement', otherwise the
 * the text ('node' is of type 'XmlContent'). */
G_CONST_RETURN gchar* m_xml_get_name(XmlNode node)
{
    return node->name;
}


/* Returns the attribute named with 'key' of 'node'. If there is no such
 * attribute then NULL is returned. */
G_CONST_RETURN gchar* m_xml_get_attribute(XmlNode node, const gchar *key)
{
    guint index;

    for (index=0; index < m_list_length(node->attributes); index+=2)
    {
        if (!strcmp(m_list_get(node->attributes, index), key))
        {
            return m_list_get(node->attributes, index + 1);
        }
    }
    return NULL;
}




/* Returns the parent node or NULL for the root node. */
XmlNode m_xml_get_parent(XmlNode node)
{
    return node->parent;
}




/* Returns the number of children of 'node'.  */
guint m_xml_get_number_of_children(XmlNode node)
{
    return m_list_length(node->children);
}


/* Returns the child at position 'index' of 'node'. If there is no such child,
 * NULL is returned. */
XmlNode m_xml_get_child(XmlNode node, guint index)
{
    return (XmlNode) m_list_get(node->children, index);
}


/* Returns the next child with name 'name' of 'node', which follows 'prev'.
 * If there is no such child, NULL is returned.
 *
 * node: XML node for which a child should be found.
 *
 * name: Name of the child which should be found. May be NULL to return the node
 *       which follows 'prev'.
 *
 * prev: Pointer to the node which is a predecessor to the node which should be
 *       found. May be NULL to find the first node which matches 'name'.
 *
 * Returns: The first node which matches 'name' and comes after 'prev'.
 *          If there is no such child, NULL is returned. */
XmlNode m_xml_get_next_child(XmlNode node, const gchar *name, XmlNode prev)
{
    guint index = 0;
    XmlNode child;
    XmlNode previous = prev;

    /* traverse all children */
    while ((child = m_xml_get_child(node, index++)) != NULL)
    {
        /* if 'child' matches 'name' and */
        if (name == NULL || !g_ascii_strcasecmp(child->name, name))
        {
            /* if there is no previous child to be found: */
            if (previous == NULL)
            {
                return child;
            }
            /* if 'previous' must be seen before we can return: */
            else if (previous == child)
            {
                previous = NULL;
            }
        }
    }
    return NULL;
}



/* Returns the content of 'node'.
 * If 'node' has more than one child only the the content of the first child
 * is returned.
 * If there is no first child of 'node' or it is not of type 'XmlContent'
 * NULL is returned. */
G_CONST_RETURN gchar* m_xml_get_content(XmlNode node)
{
    XmlNode child = m_xml_get_child(node, 0);

    if (child != NULL && m_xml_get_type(child) == XmlContent)
    {
        return  m_xml_get_name(child);
    }
    else
    {
        return NULL;
    }
}
