/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.Private.h"



/* Define a function to create a directory
 * Returns: 0 on failure, nonzero on success.
 */
#ifdef G_OS_UNIX

#include <sys/stat.h>
#include <sys/types.h>

#define _create_dir(name) (mkdir(name,0700) == 0)


#else /* not G_OS_UNIX */
#ifdef G_OS_WIN32

#include <windows.h>

#define _create_dir(name) (CreateDirectory(name,NULL) != 0)


#else /* not G_OS_WIN32 */

#error "unsupported operating system: please specify a way to create a directory"

#endif /* G_OS_WIN32 */
#endif /* G_OS_UNIX */





/* the list of all properties trees */
static XmlNode prop[NumberOfPropTypes];


/* the directory where to store user-defined matrices */
static gchar* _matrices_directory = NULL;

/* maximum size of all matrices used throughout this application */
static guint _max_matrix_size;

/* the GDK name of the highlight color for matrix cells */
static const gchar* _matrix_highlight_color;




/* Converts a string into an unsigned int. If the conversion is unsuccessful,
 * 'defaultValue' is returned. */
static guint _string_to_uint(const gchar *string, guint defaultValue)
{
    gchar *end;
    guint value;

    if (string == NULL) return FALSE;

    value = strtoul(string, &end, 10);
    return (end == string + strlen(string)) ? value : defaultValue;
}





/* Initializes the properties module. */
void prop_initialize(void)
{
    gchar *path;
    const gchar *string;

    /* load settings file */
    path = g_build_filename(m_get_application_path(), PROP_SETTINGS, NULL);
    prop[PropSettings] = m_xml_parse(path);
    g_free(path);

    /* load string file */
    string = prop_get(PropSettings, "language_file");
    path = g_build_filename(m_get_application_path(), string, NULL);
    prop[PropStrings] = m_xml_parse(path);
    g_free(path);

    /* get the matrices directory */
    string = prop_get(PropSettings, "matrices_directory");
    if (*string == '$')
    {
        _matrices_directory = g_build_filename(m_get_application_path(),
                                               string + 1,
                                               NULL);
    }
    else
    {
        _matrices_directory = g_strdup(string);
    }

    /* create the matrices directory, if it does not exist */
    _create_dir(_matrices_directory);


    /* get max matrix size */
    string = prop_get(PropSettings, "max_matrix_size");
    _max_matrix_size = _string_to_uint(string, 10);

    /* get matrix highlight color */
    _matrix_highlight_color = prop_get(PropSettings, "matrix_highlight_color");
    if (_matrix_highlight_color == NULL) _matrix_highlight_color = "#ffaa88";
}


/* Cleans up the properties module. */
void prop_clean_up(void)
{
    guint i;

    /* free all XML trees */
    for (i=0; i < NumberOfPropTypes; i++)
    {
        m_xml_free(prop[i]);
    }
}



/* Returns the value corresponding to 'key' from the properties file 'type'.
 * If there is no such key then NULL is returned. */
G_CONST_RETURN gchar* prop_get(PropType type, const gchar *key)
{
    XmlNode node = NULL;

    if (prop[type] == NULL) return NULL;

    /* traverse all 'item' nodes */
    while ((node = m_xml_get_next_child(prop[type], "item", node)) != NULL)
    {
        const gchar *attKey = m_xml_get_attribute(node, "key");

        /* if the 'key' matches the parameter then return the corresponding
         * value stored in the item's text */
        if (attKey != NULL && !strcmp(attKey, key))
        {
            XmlNode content = m_xml_get_child(node, 0);

            if (content != NULL) return m_xml_get_name(content);
        }
    }
    return NULL;
}


/* Returns the value corresponding to 'key' for 'PropStrings'.
 * If there is no such value then 'key' is returned. */
G_CONST_RETURN gchar* _T(const gchar *key)
{
    const gchar *string = prop_get(PropStrings, key);
    return (string == NULL) ? key : string;
}



/* Returns the directory where to store user-defined matrices. */
G_CONST_RETURN gchar* prop_get_matrices_directory(void)
{
    return _matrices_directory;

}



/* Returns the maximum number of rows and columns of matrices handled
 * in this program. All matrices in this program have this size. */
guint prop_get_max_matrix_size(void)
{
    return _max_matrix_size;
}



/* Returns the GDK name of the highlight color for matrix cells. */
G_CONST_RETURN gchar* prop_get_matrix_highlight_color(void)
{
    return _matrix_highlight_color;

}

