/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* holds the name of the currently selected algorithm */
static GtkTextBuffer *_nameBuffer;


/* indicates whether the pseudo-code or the description of an algorithm is
 * currently visible */
static gboolean _isCodeVisible;

/* labels the description/pseudo-code text field */
static GtkLabel *_codeLabel;

/* the description/pseudo-code text field */
static GtkTextView *_codeView;

/* holds the description/pseudo-code of the currently selected algorithm */
static GtkTextBuffer *_codeBuffer;





/* Called whenever the currently selected algorithm has changed. */
static void _on_algorithm_changed(gpointer notUsed)
{
    Algorithm algorithm = m_algorithms_get_selected();

    /* if there is no algorithm currently selected */
    if (algorithm == NULL)
    {
        gtk_text_buffer_set_text(_nameBuffer, _T("No algorithm selected."), -1);
        gtk_text_buffer_set_text(_codeBuffer,
                                 (_isCodeVisible)
                                    ? _T("No algorithm pseudo-code available.")
                                    : _T("No algorithm description available."),
                                 -1);
    }
    else    /* an algorithm is selected */
    {
        gtk_text_buffer_set_text(_nameBuffer, algorithm->name, -1);
        gtk_text_buffer_set_text(_codeBuffer,
                                 (_isCodeVisible)
                                    ? algorithm->pseudocode
                                    : algorithm->description,
                                 -1);
    }
}



/* Adapter for correctly handling the signal. */
static void _on_change_algorithm_clicked(GtkButton *button, gpointer notUsed)
{
    v_show_algorithm_dialog();
}




/* Creates the algorithm panel holding all information of an algorithm.
 * Description and pseudo-code are not displayed at the same time. Call
 * 'v_show_algorithm_description()' to activate one of them. */
void v_init_algorithm_panel(GtkBox *algorithmPanel)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkWidget *buttonPanel;
    GtkWidget *button;
    GtkTextView *view;
    gchar *string;



    /* create the title section */
    panel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(algorithmPanel, panel, FALSE, FALSE, 0);

    label = gtk_label_new("");
    string = g_strconcat("<b><big>", _T("1. Algorithm"), "</big></b>", NULL);
    gtk_label_set_markup(GTK_LABEL(label), string);
    g_free(string);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    buttonPanel = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonPanel), GTK_BUTTONBOX_START);
    gtk_box_pack_end(GTK_BOX(panel), buttonPanel, FALSE, FALSE, 6);

    button = gtk_button_new_with_mnemonic(_T("Select algorithm ..."));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    v_set_tooltip(button,
                  _T("Press this button to select another algorithm."));
    gtk_container_add(GTK_CONTAINER(buttonPanel), button);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(_on_change_algorithm_clicked), NULL);
    gtk_widget_show(button);

    gtk_widget_show(buttonPanel);

    gtk_widget_show(panel);



    /*  create the name section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(algorithmPanel, panel, FALSE, FALSE, 6);

    label = gtk_label_new(_T("Algorithm name:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 550, 1);
    _nameBuffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(view));
    gtk_text_buffer_set_text(_nameBuffer, _T("No algorithm selected."), -1);

    gtk_widget_show(panel);



    /*  create the description/pseudo-code section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(algorithmPanel, panel, FALSE, FALSE, 6);

    label = gtk_label_new(_T("Algorithm pseudo-code:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 550, 13);
    v_set_monospace_font(view, TRUE);
    _codeBuffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(view));
    gtk_text_buffer_set_text(_codeBuffer,
                             _T("No algorithm pseudo-code available."),
                             -1);

    gtk_widget_show(panel);


    /* store references to needed components */
    _codeLabel = GTK_LABEL(label);
    _codeView = GTK_TEXT_VIEW(view);
    _isCodeVisible = TRUE;



    /* add callback function to stay informed whenever the algorithm changes */
    m_algorithms_add_hook(OnActiveItemHasChanged, _on_algorithm_changed);
}



/* Used to switch to either the description or the pseudo-code of an
 * algorithm. Only of of them can be displayed at a time. */
void v_show_algorithm_description(gboolean showDescription)
{
    _isCodeVisible = !showDescription;

    /* change the font accordingly */
    v_set_monospace_font(_codeView, _isCodeVisible);

    /* only used to set the current value */
    _on_algorithm_changed(NULL);
}
