/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* the list view containing all available algorithms */
static GtkWidget *_listView;

/* describes the selected items of the list view */
static GtkTreeSelection *_selection;


/* holds the description of the currently selected algorithm */
static GtkTextBuffer *_descriptionBuffer;

/* holds the pseudocode of the currently selected algorithm */
static GtkTextBuffer *_codeBuffer;



/* Called whenever the selected item in the list view has changed. */
static void _on_active_item_changed(GtkTreeSelection *selection,                                                     gpointer notUsed)
{
    guint selected = v_get_selected_list_item_index(selection);
    Algorithm algorithm = m_algorithms_get(selected);

    /* update the description */
    gtk_text_buffer_set_text(_descriptionBuffer,
                             (algorithm == NULL)
                                ? _T("No algorithm description available")
                                : algorithm->description,
                             -1);

    /* update the pseudo-code */
    gtk_text_buffer_set_text(_codeBuffer,
                             (algorithm == NULL)
                                ? _T("No algorithm pseudo-code available")
                                : algorithm->pseudocode,
                             -1);
}



/* Creates the list view holding all available algorithms. */
static void _create_list(GtkBox *listPanel)
{
    GtkWidget *label;
    GtkListStore *store;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    GtkTreeIter iter;
    guint i;

    GtkWidget *scroll;



    /* create label for the list view */
    label = gtk_label_new(_T("Available algorithms:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(listPanel, label, FALSE, FALSE, 6);
    gtk_widget_show(label);



    /* collect all algorithms */
    store = gtk_list_store_new(2, G_TYPE_UINT, G_TYPE_STRING);
    for (i=0; i < m_algorithms_length(); i++)
    {
        Algorithm algorithm = m_algorithms_get(i);

        gtk_list_store_append(store, &iter);
        gtk_list_store_set(store, &iter, 0, i, 1, algorithm->name, -1);
    }



    /* create the scroll window */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_widget_set_size_request(scroll, 200, 13 * v_get_line_height() + 6);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
                                        GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(listPanel, scroll, FALSE, FALSE, 0);



    /* create the list view */
    _listView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("", renderer,
                                                      "text", 1, NULL);

    gtk_tree_view_append_column(GTK_TREE_VIEW(_listView), column);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(_listView), FALSE);

    _selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(_listView));
    g_signal_connect(G_OBJECT(_selection), "changed",
                     G_CALLBACK(_on_active_item_changed), NULL);

    gtk_container_add(GTK_CONTAINER(scroll), _listView);


    /* show the list view */
    gtk_widget_show(_listView);
    gtk_widget_show(scroll);
}



/* Creates the right side of the dialog containing the description and the
 * pseudo-code text fields. */
static void _create_info(GtkBox *infoPanel)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkTextView *view;


    /*  create the description section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(infoPanel), panel, FALSE, FALSE, 0);

    label = gtk_label_new(_T("Algorithm description:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 550, 4);
    _descriptionBuffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(view));
    gtk_text_buffer_set_text(_descriptionBuffer,
                             _T("No algorithm description available."),
                             -1);

    gtk_widget_show(panel);


    /*  create the pseudo-code section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(infoPanel), panel, FALSE, FALSE, 6);

    label = gtk_label_new(_T("Algorithm pseudo-code:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 550, 13);
    v_set_monospace_font(view, TRUE);
    _codeBuffer = GTK_TEXT_BUFFER(gtk_text_view_get_buffer(view));
    gtk_text_buffer_set_text(_codeBuffer,
                             _T("No algorithm pseudo-code available."),
                             -1);

    gtk_widget_show(panel);
}



/* Creates the main panel holding all other components of the dialog. */
static void _create_main_panel(GtkWidget *rootPanel)
{
    GtkWidget *mainPanel;
    GtkWidget *helpLabel;
    GtkWidget *listPanel;
    GtkWidget *infoPanel;


    /* create a description what to do in this dialog */
    helpLabel = gtk_label_new(
                    _T("Please select an algorithm from the list below."));
    gtk_misc_set_alignment(GTK_MISC(helpLabel), 0, 0);
    gtk_misc_set_padding(GTK_MISC(helpLabel), 6, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), helpLabel, FALSE, FALSE, 6);
    gtk_widget_show(helpLabel);


    /* create the main panel holding the list view and the info section */
    mainPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), mainPanel, FALSE, FALSE, 6);


    /* create the list view */
    listPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), listPanel, FALSE, FALSE, 6);

    _create_list(GTK_BOX(listPanel));

    gtk_widget_show(listPanel);


    /* create the info section */
    infoPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), infoPanel, FALSE, FALSE, 6);

    _create_info(GTK_BOX(infoPanel));

    gtk_widget_show(infoPanel);


    gtk_widget_show(mainPanel);
}



/* Creates a warning dialog. */
static void _show_warning_dialog(void)
{
    GtkWidget *dialog;
    gchar *string;


    /* construct the warning text */
    string = g_strconcat(_T("Selecting an algorithm is not possible because "
                            "no algorithm was found in the algorithms' "
                            "directory."),
                         "\n\n",
                         _T("Please make sure there is at least one "
                            "algorithm file in the algorithms' "
                            "directory."),
                         "\n",
                         _T("Note that new algorithms are only available "
                            "after a restart of this application."),
                         NULL);


    /* create the dialog */
    dialog = gtk_message_dialog_new(
                        v_get_main_window(),
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_MESSAGE_WARNING,
                        GTK_BUTTONS_NONE,
                        string);

    g_free(string);

    gtk_window_set_title(GTK_WINDOW(dialog), _T("No algorithm was found"));
    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
                           _T("OK"), GTK_RESPONSE_ACCEPT,
                           NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);


    /* show the dialog and destroy it after the user pressed "OK" */
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
}



/* Shows a dialog in which the user can choose an algorithm from a list.
 * Description and pseudo-code are displayed for the currently selected
 * algorithm. */
void v_show_algorithm_dialog(void)
{
    static GtkWidget *dialog = NULL;
    gint result;


    /* if there is at least one algorithm, create the normal dialog */
    if (m_algorithms_length() > 0)
    {
        /* create the dialog only once */
        if (dialog == NULL)
        {
        GtkTreePath *path;

            dialog =  gtk_dialog_new_with_buttons(
                                _T("Select an algorithm"),
                                v_get_main_window(),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                _T("OK"),
                                GTK_RESPONSE_ACCEPT,
                                _T("Cancel"),
                                GTK_RESPONSE_REJECT,
                                NULL);
            gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
            gtk_dialog_set_default_response(GTK_DIALOG(dialog),
                                            GTK_RESPONSE_ACCEPT);

            /* create the dialog contents */
            _create_main_panel(GTK_DIALOG(dialog)->vbox);


            /* select the first algorithm from the list */
            path = gtk_tree_path_new_from_indices(0, -1);
            gtk_tree_selection_select_path(_selection, path);
            gtk_tree_path_free(path);
        }
        else    /* if the dialog is already created: */
        {
            /* focus the list view */
            gtk_widget_grab_focus(_listView);
        }
    }
    else    /* show warning dialog if there is no algorithm */
    {
        _show_warning_dialog();
        return;
    }


    /* show the dialog */
    result = gtk_dialog_run(GTK_DIALOG(dialog));


    /* select an algorithm if the user pressed "OK" */
    if (result == GTK_RESPONSE_ACCEPT)
    {
        m_algorithms_select(v_get_selected_list_item_index(_selection));
    }


    /* do not destroy the dialog, reusing it is surely faster */
    gtk_widget_hide(dialog);
}
