/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* the button with the label "Select initial matrix ..." */
static GtkWidget *_changeButton;

/* the button with the label "Compute result" */
static GtkWidget *_computeButton;

/* the button with the label "Compute all steps" */
static GtkWidget *_stepwiseButton;



/* the button to show the next computation result */
static GtkWidget *_nextButton;

/* the button to show the previous computation result */
static GtkWidget *_prevButton;



/* the matrix items of the left matrix
 * there is no explicit call to 'g_free()' ! */
static GtkWidget ***_leftMatrixItem;

/* the matrix items of the right matrix
 * there is no explicit call to 'g_free()' ! */
static GtkWidget ***_rightMatrixItem;


/* the label of the left matrix */
static GtkWidget *_leftLabel;

/* the label of the right matrix */
static GtkWidget *_rightLabel;



/* used in placing matrices centered */
static guint _offset;



/* the normal color of matrix items in contrast to highlighted items */
static GdkColor _normalColor;






/* Updates one the visible matrices.
 *
 * matrix: The data to display.
 * compare: Matrix to compare 'matrix' with. If some item is different,
 *          it will be highlighted.
 * entries: Entry widgets to display the corresponding matrix items.
 */
static void _show_matrix(Matrix matrix, Matrix compare, GtkWidget ***entries)
{
    guint row, column, size = prop_get_max_matrix_size();
    GdkColor highlightColor;

    /* get the highlight color */
    gdk_color_parse(prop_get_matrix_highlight_color(), &highlightColor);


    if (matrix != NULL)
    {
        for (row=0; row < size; row++)
        {
            for (column=0; column < size; column++)
            {
                /* the items of 'matrix' are displayed centered in the
                 * table of 'entries' */
                if (row >= _offset && row < matrix->size + _offset &&
                    column >= _offset && column < matrix->size + _offset)
                {
                    SemiringItem si;
                    gchar *item;

                    /* transform the stored item into a string */
                    si = matrix->matrix[row - _offset][column - _offset];
                    item = matrix->semiring->to_string(si);

                    /* add a tooltip if the item is to long */
                    if (strlen(item) > 5)
                    {
                        v_set_tooltip(entries[row][column], item);

                        item[3] = '.';
                        item[4] = '.';
                        item[5] = '.';
                        item[6] = 0;
                    }
                    else
                    {
                        v_set_tooltip(entries[row][column], NULL);
                    }

                    /* display this string */
                    gtk_entry_set_text(GTK_ENTRY(entries[row][column]), item);
                    gtk_widget_show(entries[row][column]);

                    /* colorize the item */
                    if (compare != NULL)
                    {
                        SemiringItem sic;
                        GdkColor *color;

                        sic = compare->matrix[row-_offset][column-_offset];

                        color = (matrix->semiring->equals(si, sic))
                                    ? &_normalColor
                                    : &highlightColor;

                        gtk_widget_modify_base(entries[row][column],
                                               GTK_STATE_NORMAL,
                                               color);
                    }

                    g_free(item);
                }
                else    /* hide all entries which have no valid value */
                {
                    gtk_widget_hide(entries[row][column]);
                }
            }
        }
    }
}



/* Called whenever the currently selected semiring has changed. */
static void _on_semiring_changed(gpointer notUsed)
{
    gboolean enable = (m_semirings_get_selected() != NULL);

    gtk_widget_set_sensitive(_changeButton, enable);
}


/* Called whenever the currently selected matrix has changed. */
static void _on_matrix_changed(gpointer notUsed)
{
    Matrix matrix = m_matrices_get_selected();
    guint row, column, size = prop_get_max_matrix_size();

    /* disable the arrow buttons */
    gtk_widget_set_sensitive(_prevButton, FALSE);
    gtk_widget_set_sensitive(_nextButton, FALSE);


    if (matrix == NULL)
    {
        /* disable the "Compute XXX" buttons */
        gtk_widget_set_sensitive(_computeButton, FALSE);
        gtk_widget_set_sensitive(_stepwiseButton, FALSE);

        /* clear the matrix labels */
        gtk_label_set_text(GTK_LABEL(_leftLabel), "");
        gtk_label_set_text(GTK_LABEL(_rightLabel), "");
    }
    else
    {
        /* enable the "Compute XXX" buttons */
        gtk_widget_set_sensitive(_computeButton, TRUE);
        gtk_widget_set_sensitive(_stepwiseButton, TRUE);

        /* set the matrix labels */
        gtk_label_set_text(GTK_LABEL(_leftLabel), _T("Initial matrix"));
        gtk_label_set_text(GTK_LABEL(_rightLabel), "");


        /* re-compute the offset used to center the matrices */
        _offset = (prop_get_max_matrix_size() - matrix->size) / 2;

        /* show the left matrix */
        _show_matrix(matrix, NULL, _leftMatrixItem);
    }


    /* clear the right and (if needed) the left matrix */
    for (row=0; row < size; row++)
    {
        for (column=0; column < size; column++)
        {
            if (matrix == NULL)
            {
                gtk_entry_set_text(GTK_ENTRY(_leftMatrixItem[row][column]), "");
                gtk_widget_show(_leftMatrixItem[row][column]);

                gtk_entry_set_text(GTK_ENTRY(_rightMatrixItem[row][column]),
                                   "");
                gtk_widget_show(_rightMatrixItem[row][column]);
            }
            else {
                if (row >= _offset && row < matrix->size + _offset &&
                     column >= _offset && column < matrix->size + _offset)
                {
                    gtk_entry_set_text(GTK_ENTRY(_rightMatrixItem[row][column]),
                                       "");
                    gtk_widget_show(_rightMatrixItem[row][column]);
                }
                else
                {
                    gtk_widget_hide(_rightMatrixItem[row][column]);
                }
            }

            gtk_widget_modify_base(_rightMatrixItem[row][column],
                                   GTK_STATE_NORMAL,
                                   &_normalColor);
        }
    }
}



/* Adapter for correctly handling the signal. */
static void _on_change_matrix_clicked(GtkButton *button, gpointer notUsed)
{
    v_show_matrix_dialog();
}


/* Adapter for correctly handling the signal. */
static void _on_compute_clicked(GtkButton *button, gpointer notUsed)
{
    v_compute(FALSE);
}


/* Adapter for correctly handling the signal. */
static void _on_compute_stepwise_clicked(GtkButton *button, gpointer notUsed)
{
    v_compute(TRUE);
}


/* Activate the previous computation step. */
static void _on_prev_clicked(GtkButton *button, gpointer notUsed)
{
    m_computation_select_previous();

    /* enable/disable the arrow buttons as needed */
    gtk_widget_set_sensitive(_prevButton, m_computation_has_previous());
    gtk_widget_set_sensitive(_nextButton, TRUE);
}


/* Activate the next computation step. */
static void _on_next_clicked(GtkButton *button, gpointer notUsed)
{
    m_computation_select_next();

    /* enable/disable the arrow buttons as needed */
    gtk_widget_set_sensitive(_prevButton, TRUE);
    gtk_widget_set_sensitive(_nextButton, m_computation_has_next());
}



/* Called whenever the the visible computed matrices have changed. */
static void _on_computed_matrices_changed(gpointer notUsed)
{
    guint index = m_computation_get_selected_index();

    /* retrieve the two matrices which should be displayed */
    Matrix left = m_computation_get_selected();
    Matrix right = m_computation_get_2nd_selected();

    /* show both matrices */
    _show_matrix(left, NULL, _leftMatrixItem);
    _show_matrix(right, left, _rightMatrixItem);


    /* update the left matrix label */
    if (m_computation_has_previous())
    {
        gchar *text = g_strdup_printf("%d. %s", index, _T("step"));
        gtk_label_set_text(GTK_LABEL(_leftLabel), text);
        g_free(text);
    }
    else
    {
        gtk_label_set_text(GTK_LABEL(_leftLabel), _T("Initial matrix"));
    }


    /* update the right matrix label */
    if (m_computation_has_next())
    {
        gchar *text = g_strdup_printf("%d. %s", index+1, _T("step"));
        gtk_label_set_text(GTK_LABEL(_rightLabel), text);
        g_free(text);
    }
    else
    {
        gtk_label_set_text(GTK_LABEL(_rightLabel), _T("Final matrix"));
    }
}




/* Create the title section with the three buttons "Select initial matrix ...",
 * and "Compute XXX". */
static void _create_title_section(GtkBox *parent)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkWidget *buttonPanel;

    gchar *string;


    /* create the title section */
    panel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(parent, panel, FALSE, FALSE, 12);

    label = gtk_label_new("");
    string = g_strconcat("<b><big>", _T("3. Computation"), "</big></b>", NULL);
    gtk_label_set_markup(GTK_LABEL(label), string);
    g_free(string);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 12);
    gtk_widget_show(label);

    buttonPanel = gtk_hbutton_box_new();
    gtk_box_set_spacing(GTK_BOX(buttonPanel), 12);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonPanel), GTK_BUTTONBOX_START);
    gtk_box_pack_end(GTK_BOX(panel), buttonPanel, FALSE, FALSE, 12);

    _changeButton = gtk_button_new_with_mnemonic(_T("Select initial matrix ..."));
    gtk_widget_set_sensitive(_changeButton, FALSE);
    v_set_tooltip(_changeButton,
                  _T("Press this button to select another initial matrix."));
    gtk_container_add(GTK_CONTAINER(buttonPanel), _changeButton);
    g_signal_connect(G_OBJECT(_changeButton), "clicked",
                     G_CALLBACK(_on_change_matrix_clicked), NULL);
    gtk_widget_show(_changeButton);

    _computeButton = gtk_button_new_with_mnemonic(_T("Compute result"));
    gtk_widget_set_sensitive(_computeButton, FALSE);
    v_set_tooltip(_computeButton,
                  _T("Press this button to compute the result only."));
    gtk_container_add(GTK_CONTAINER(buttonPanel), _computeButton);
    g_signal_connect(G_OBJECT(_computeButton), "clicked",
                     G_CALLBACK(_on_compute_clicked), NULL);
    gtk_widget_show(_computeButton);

    _stepwiseButton = gtk_button_new_with_mnemonic(_T("Compute all steps"));
    gtk_widget_set_sensitive(_stepwiseButton, FALSE);
    v_set_tooltip(_stepwiseButton,
                  _T("Press this button to compute the result and "
                     "intermediate steps."));
    gtk_container_add(GTK_CONTAINER(buttonPanel), _stepwiseButton);
    g_signal_connect(G_OBJECT(_stepwiseButton), "clicked",
                     G_CALLBACK(_on_compute_stepwise_clicked), NULL);
    gtk_widget_show(_stepwiseButton);


    gtk_widget_show(buttonPanel);

    gtk_widget_show(panel);
}



/* Create the matrix section showing two matrices and a left arrow button and
 * a right arrow button. */
static void _create_matrix_section(GtkBox *parent)
{
    GtkWidget *matrixPanel;
    GtkWidget *globalAlign;
    GtkWidget *buttonAlign;
    GtkWidget *arrow;
    GtkWidget *panel;
    GtkWidget *matrix;
    GtkWidget *emptyPanel;

    GtkStyle *style;

    guint row, column, size = prop_get_max_matrix_size();



    /* place the matrices and buttons horizontally centered */
    globalAlign = gtk_alignment_new(0.5, 0.5, 0, 0);
    gtk_box_pack_start(parent, globalAlign, FALSE, FALSE, 6);

    matrixPanel = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(globalAlign), matrixPanel);



    /* create the left arrow button */
    buttonAlign = gtk_alignment_new(0, 0.5, 0, 0);
    gtk_box_pack_start(GTK_BOX(matrixPanel), buttonAlign, FALSE, FALSE, 6);

    _prevButton = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
    gtk_container_add(GTK_CONTAINER(_prevButton), arrow);
    v_set_tooltip(_prevButton,
                  _T("Press this button to show the previous computation "
                     "step."));
    gtk_widget_set_sensitive(_prevButton, FALSE);
    gtk_container_add(GTK_CONTAINER(buttonAlign), _prevButton);
    g_signal_connect(G_OBJECT(_prevButton), "clicked",
                     G_CALLBACK(_on_prev_clicked), NULL);
    gtk_widget_show(arrow);
    gtk_widget_show(_prevButton);
    gtk_widget_show(buttonAlign);




    panel = gtk_vbox_new(FALSE, 12);
    gtk_box_pack_start(GTK_BOX(matrixPanel), panel, FALSE, FALSE, 12);

    matrix = gtk_table_new(size, size, TRUE);
    gtk_box_pack_start(GTK_BOX(panel), matrix, FALSE, FALSE, 0);


    /* create the left matrix */
    _leftMatrixItem = g_new0(GtkWidget**, size);
    for (row=0; row < size; row++)
    {
        _leftMatrixItem[row] = g_new0(GtkWidget*, size);
        for (column=0; column < size; column++)
        {
            GtkEntry *entry;

            _leftMatrixItem[row][column] = gtk_entry_new();
            entry = GTK_ENTRY(_leftMatrixItem[row][column]);

            gtk_entry_set_width_chars(entry, 5);
            gtk_entry_set_editable(entry, FALSE);
            gtk_table_attach_defaults(GTK_TABLE(matrix), GTK_WIDGET(entry),
                                      column, column+1, row, row+1);

            gtk_widget_show(_leftMatrixItem[row][column]);
        }
    }


    /* create the left matrix label under the matrix */
    _leftLabel = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(_leftLabel), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(panel), _leftLabel, FALSE, FALSE, 0);
    gtk_widget_show(_leftLabel);


    gtk_widget_show(matrix);
    gtk_widget_show(panel);



    /* create an empty panel (for a better layout) */
    emptyPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(matrixPanel), emptyPanel, FALSE, FALSE, 12);
    gtk_widget_show(emptyPanel);




    panel = gtk_vbox_new(FALSE, 12);
    gtk_box_pack_start(GTK_BOX(matrixPanel), panel, FALSE, FALSE, 12);

    matrix = gtk_table_new(size, size, TRUE);
    gtk_box_pack_start(GTK_BOX(panel), matrix, FALSE, FALSE, 0);


    /* create the right matrix */
    _rightMatrixItem = g_new0(GtkWidget**, size);
    for (row=0; row < size; row++)
    {
        _rightMatrixItem[row] = g_new0(GtkWidget*, size);
        for (column=0; column < size; column++)
        {
            GtkEntry *entry;

            _rightMatrixItem[row][column] = gtk_entry_new();
            entry = GTK_ENTRY(_rightMatrixItem[row][column]);

            gtk_entry_set_width_chars(entry, 5);
            gtk_entry_set_editable(entry, FALSE);
            gtk_table_attach_defaults(GTK_TABLE(matrix), GTK_WIDGET(entry),
                                      column, column+1, row, row+1);

            gtk_widget_show(_rightMatrixItem[row][column]);
        }
    }


    /* create the right matrix label under the matrix */
    _rightLabel = gtk_label_new("");
    gtk_misc_set_alignment(GTK_MISC(_rightLabel), 0.5, 0);
    gtk_box_pack_start(GTK_BOX(panel), _rightLabel, FALSE, FALSE, 0);
    gtk_widget_show(_rightLabel);


    gtk_widget_show(matrix);
    gtk_widget_show(panel);



    /* create the right arrow button */
    buttonAlign = gtk_alignment_new(0, 0.5, 0, 0);
    gtk_box_pack_start(GTK_BOX(matrixPanel), buttonAlign, FALSE, FALSE, 6);

    _nextButton = gtk_button_new();
    arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
    gtk_container_add(GTK_CONTAINER(_nextButton), arrow);
    v_set_tooltip(_nextButton,
                  _T("Press this button to show the next computation step."));
    gtk_widget_set_sensitive(_nextButton, FALSE);
    gtk_container_add(GTK_CONTAINER(buttonAlign), _nextButton);
    g_signal_connect(G_OBJECT(_nextButton), "clicked",
                     G_CALLBACK(_on_next_clicked), NULL);
    gtk_widget_show(arrow);
    gtk_widget_show(_nextButton);
    gtk_widget_show(buttonAlign);


    gtk_widget_show(matrixPanel);
    gtk_widget_show(globalAlign);




    /* retrieve the normal color of a matrix entry */
    style = gtk_widget_get_style(_leftMatrixItem[0][0]);
    _normalColor = style->base[GTK_STATE_NORMAL];
}



/* Creates the matrix panel holding all information of the computation and
 * its matrices. */
void v_init_matrix_panel(GtkBox *bottomPanel)
{
    /* create the visible components */
    _create_title_section(bottomPanel);
    _create_matrix_section(bottomPanel);


    /* add callback functions to stay informed whenever the selected semiring
     * or matrix changes */
    m_semirings_add_hook(OnActiveItemHasChanged, _on_semiring_changed);
    m_matrices_add_hook(OnActiveItemHasChanged, _on_matrix_changed);
    m_computation_add_hook(OnActiveItemHasChanged,
                           _on_computed_matrices_changed);
}



/* Executes the computation.
 *
 * stepwise : TRUE if all intermediate steps should also be computed,
 *            otherwise FALSE.
 */
void v_compute(gboolean stepwise)
{
    Computation computation;


    /* compute all matrices */
    m_computation_execute(stepwise);
    computation = m_computation_get_last();

    /* enable/disable buttons as needed */
    gtk_widget_set_sensitive(_computeButton,
                             !(computation == OnlyResultComputed));
    gtk_widget_set_sensitive(_stepwiseButton,
                             !(computation == AllStepsComputed));
    gtk_widget_set_sensitive(_nextButton, m_computation_has_next());

    /* update the matrix section */
    _on_computed_matrices_changed(NULL);
}
