/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"




/* describes the selected items of the list view */
static GtkTreeSelection *_selection;

/* holds the items visible in the list view */
static GtkListStore *_listStore;

/* the list view itself */
static GtkWidget *_listView;


/* the button to remove a matrix */
static GtkWidget *_removeButton;


/* the matrix size text field (of type GtkEntry*) */
static GtkWidget *_matrixSizeEntry;


/* 2-dimensional list of all matrix entry fields (each of type GtkEntry*)
 * there is no explicit call to 'g_free()' ! */
static GtkWidget ***_matrixItem;




/* Called whenever the currently selected semiring has changed. */
static void _on_semiring_changed(gpointer notUsed)
{
    GtkTreeIter iter;
    guint i;

    /* clear the matrices list */
    gtk_list_store_clear(_listStore);

    /* and refill it with the currently possible matrices */
    for (i=0; i < m_matrices_length(); i++)
    {
        Matrix matrix = m_matrices_get(i);

        gtk_list_store_append(_listStore, &iter);
        gtk_list_store_set(_listStore, &iter, 0, i, 1, matrix->name, -1);
    }
}



/* Called whenever the selected item in the list view has changed. */
static void _on_active_item_changed(GtkTreeSelection *selection,
                                    gpointer notUsed)
{
    guint selected = v_get_selected_list_item_index(selection);
    Matrix matrix = m_matrices_get(selected);
    guint row, column;


    /* if there is no item selected, disable the remove button */
    gtk_widget_set_sensitive(_removeButton, matrix != NULL);


    /* update the size field */
    gtk_widget_set_sensitive(_matrixSizeEntry, matrix != NULL);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(_matrixSizeEntry),
                              (matrix != NULL)
                                ? matrix->size
                                : prop_get_max_matrix_size());


    /* update the matrix */
    for (row=0; row < prop_get_max_matrix_size(); row++)
    {
        for (column=0; column < prop_get_max_matrix_size(); column++)
        {
            gchar *item;

            if (matrix != NULL)
            {
                /* transform the stored item into a string */
                item = matrix->semiring->to_string(matrix->matrix[row][column]);

                /* display this string */
                gtk_entry_set_text(GTK_ENTRY(_matrixItem[row][column]), item);
                gtk_widget_set_sensitive(_matrixItem[row][column], TRUE);

                /* add a tooltip if the item is to long */
                if (strlen(item) > 5)
                {
                    v_set_tooltip(_matrixItem[row][column], item);
                }

                g_free(item);
            }
            else
            {
                gtk_entry_set_text(GTK_ENTRY(_matrixItem[row][column]), "");
                gtk_widget_set_sensitive(_matrixItem[row][column], FALSE);
            }
        }
    }
}



/* Checks if 'name' is a valid matrix name. If name is empty, if it contains
 * other characters than numbers and letter or if there exists already a
 * matrix with the same name, 'name' is considered invalid. */
static gboolean _is_invalid(const gchar *name)
{
    guint i;
    const gchar *p;


    /* do not allow empty names */
    if (strlen(name) == 0) return TRUE;

    /* check if 'name' only consists of allowed characters */
    for (p=name; *p != 0; p++)
    {
        if (!g_ascii_isalnum(*p) && *p != '_') return TRUE;
    }

    /* look up there is already such a name ofr a matrix */
    for (i=0; i < m_matrices_length(); i++)
    {
        if (!strcmp(m_matrices_get(i)->name, name))
        {
            return TRUE;
        }
    }

    return FALSE;
}


/* Called whenever the "New" button was pressed. */
static void _on_new_matrix_clicked(GtkButton *widget, gpointer notUsed)
{
    GtkWidget *dialog;
    GtkWidget *rootPanel;
    GtkWidget *panel;
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *entry;
    gint result;

    GtkTreeIter iter;


    /* construct a dialog where the user can enter the name of the matrix */
    dialog =  gtk_dialog_new_with_buttons(
                        _T("Matrix name"),
                        v_get_main_window(),
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        _T("OK"),
                        GTK_RESPONSE_ACCEPT,
                        _T("Cancel"),
                        GTK_RESPONSE_REJECT,
                        NULL);
    gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);


    rootPanel = GTK_DIALOG(dialog)->vbox;

    panel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), panel, FALSE, FALSE, 12);

    table = gtk_table_new(2, 1, TRUE);
    gtk_table_set_row_spacings(GTK_TABLE(table), 12);
    gtk_table_set_col_spacings(GTK_TABLE(table), 12);
    gtk_box_pack_start(GTK_BOX(panel), table, FALSE, FALSE, 12);

    label = gtk_label_new(_T("Please enter the name of the new matrix:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
    gtk_widget_show(label);

    entry = gtk_entry_new();
    gtk_table_attach_defaults(GTK_TABLE(table), entry, 0, 1, 1, 2);
    gtk_widget_show(entry);

    gtk_widget_show(table);
    gtk_widget_show(panel);


    /* show the dialog */
    result = gtk_dialog_run(GTK_DIALOG(dialog));


    /* if the user pressed "OK", a new matrix will be created */
    if (result == GTK_RESPONSE_ACCEPT)
    {
        const gchar *newName = gtk_entry_get_text(GTK_ENTRY(entry));

        /* test if the new name is invalid */
        if (_is_invalid(newName))
        {
            GtkWidget *errorDialog;
            gint result;
            gchar *string;

            /* construct the error text */
            string = g_strconcat(_T("The entered name is not allowed."),
                                 "\n\n",
                                 _T("Please retry choosing another name "
                                    "considering that a matrix name must not "
                                    "be empty, should only consist of letters, "
                                    "numbers and '_' and that it must not "
                                    "already exist."),
                                 NULL);

            /* create the dialog */
            errorDialog = gtk_message_dialog_new(
                                v_get_main_window(),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                GTK_MESSAGE_ERROR,
                                GTK_BUTTONS_NONE,
                                string);
            g_free(string);

            gtk_window_set_title(GTK_WINDOW(errorDialog), _T("Error"));
            gtk_dialog_add_buttons(GTK_DIALOG(errorDialog),
                                   _T("OK"), GTK_RESPONSE_ACCEPT,
                                   NULL);
            gtk_dialog_set_default_response(GTK_DIALOG(errorDialog),
                                            GTK_RESPONSE_ACCEPT);

            /* show the dialog and destroy it after the user pressed "OK" */
            result = gtk_dialog_run(GTK_DIALOG(errorDialog));
            gtk_widget_destroy(errorDialog);
        }
        else    /* the new name is valid */
        {
            Matrix m;
            guint index;
            GtkTreePath *path;

            /* create a new matrix */
            m_matrices_add_new_matrix();

            /* set the previously entered name as the name of the new matrix */
            index = m_matrices_length() - 1;
            m = m_matrices_get(index);
            g_free(m->name);
            m->name = g_strdup(newName);

            /* show the new matrix in the list view */
            gtk_list_store_append(_listStore, &iter);
            gtk_list_store_set(_listStore, &iter, 0, index, 1, m->name, -1);


            /* select the current semiring from the list */
            path = gtk_tree_path_new_from_indices(index, -1);
            gtk_tree_selection_select_path(_selection, path);
            gtk_tree_path_free(path);
        }
    }


    /* destroy the matrix */
    gtk_widget_destroy(dialog);


    /* set the focus back to the list view */
    gtk_widget_grab_focus(_listView);
}



/* Called whenever the "Remove" button was pressed. */
static void _on_remove_matrix_clicked(GtkButton *widget, gpointer notUsed)
{
    guint selected = v_get_selected_list_item_index(_selection);
    Matrix matrix = m_matrices_get(selected);
    GtkTreeIter iter;
    guint i;

    GtkWidget *dialog;
    gint result;
    gchar *string;


    /* construct the confirmation text */
    string = g_strconcat(_T("Should the following matrix really be removed?"),
                         "\n",
                         _T("Keep in mind that this process can not be "
                            "undone."),
                         "\n\n",
                         _T("Matrix name:"),
                         "\t",
                         matrix->name,
                         NULL);


    /* create the dialog */
    dialog = gtk_message_dialog_new(
                        v_get_main_window(),
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_MESSAGE_QUESTION,
                        GTK_BUTTONS_NONE,
                        string);
    g_free(string);

    gtk_window_set_title(GTK_WINDOW(dialog), _T("Confirmation"));
    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
                           _T("Yes"), GTK_RESPONSE_ACCEPT,
                           _T("No"), GTK_RESPONSE_REJECT,
                           NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);


    /* show the dialog and destroy it after the user pressed either "OK" or
     * "Cancel" */
    result = gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);



    /* if the user pressed "OK", the selected matrix will be removed */
    if (result == GTK_RESPONSE_ACCEPT)
    {
        m_matrices_remove_matrix(selected);

        /* remove the matrix from the list view */
        gtk_list_store_clear(_listStore);
        for (i=0; i < m_matrices_length(); i++)
        {
            Matrix matrix = m_matrices_get(i);

            gtk_list_store_append(_listStore, &iter);
            gtk_list_store_set(_listStore, &iter, 0, i, 1, matrix->name, -1);
        }

        /* if there is no matrix after all, disable the "Remove" buttono */
        if (m_matrices_length() == 0)
        {
            gtk_widget_set_sensitive(_removeButton, FALSE);
        }
    }


    /* set the focus back to the list view */
    gtk_widget_grab_focus(_listView);
}



/* Called whenever the matrix size field has changed. */
static void _on_matrix_size_changed(GtkSpinButton *spin, gpointer notUsed)
{
    guint selected = v_get_selected_list_item_index(_selection);
    Matrix matrix = m_matrices_get(selected);

    guint row, column;
    gdouble size = gtk_spin_button_get_value(spin);

    /* store the matrix size */
    if (matrix != NULL && matrix->size != size)
    {
        matrix->size = size;
        m_matrices_selected_modified();
    }

    /* update the visible matrix fields */
    for (row=0; row < prop_get_max_matrix_size(); row++)
    {
        for (column=0; column < prop_get_max_matrix_size(); column++)
        {
            if (row < size && column < size)
            {
                gtk_widget_show(_matrixItem[row][column]);
            }
            else
            {
                gtk_widget_hide(_matrixItem[row][column]);
            }
        }
    }
}



/* Called whenever editing of a matrix field has finished. */
static gboolean _on_entry_lost_focus(GtkWidget *widget, GdkEventFocus *event,
                                     gpointer user_data)
{
    /* extract the position of the changed matrix field from 'user_data' */
    guint compressed = GPOINTER_TO_UINT(user_data);
    guint row = compressed / prop_get_max_matrix_size();
    guint column = compressed % prop_get_max_matrix_size();

    const gchar *text;
    guint read;

    guint selected = v_get_selected_list_item_index(_selection);
    Matrix matrix = m_matrices_get(selected);
    SemiringItem srItem;

    if (matrix != NULL)
    {
        /* transform the string of the matrix field into a semiring item */
        text = gtk_entry_get_text(GTK_ENTRY(_matrixItem[row][column]));
        srItem = matrix->semiring->from_string(text, &read);

        /* if transformation was completely possible, store the semiring item */
        if (read > 0 && read == strlen(text))
        {
            matrix->matrix[row][column] = srItem;
            m_matrices_selected_modified();
        }
        else    /* an error occurred during transformation */
        {
            /* undo the user input  */
            gchar *text;
            text = matrix->semiring->to_string(matrix->matrix[row][column]);
            gtk_entry_set_text(GTK_ENTRY(_matrixItem[row][column]), text);
            g_free(text);
        }
    }

    return FALSE;
}




/* Creates the list view holding all selectable matrices. */
static void _create_list(GtkBox *listPanel)
{
    GtkWidget *label;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    GtkTreeIter iter;
    guint i;

    GtkWidget *scroll;

    GtkWidget *buttonBox;
    GtkWidget *button;



    /* create label for the list view */
    label = gtk_label_new(_T("Available matrices:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(listPanel, label, FALSE, FALSE, 6);
    gtk_widget_show(label);



    /* collect all matrices */
    _listStore = gtk_list_store_new(2, G_TYPE_UINT, G_TYPE_STRING);
    for (i=0; i < m_matrices_length(); i++)
    {
        Matrix matrix = m_matrices_get(i);

        gtk_list_store_append(_listStore, &iter);
        gtk_list_store_set(_listStore, &iter, 0, i, 1, matrix->name, -1);
    }



    /* create the scroll window */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_widget_set_size_request(scroll, 200, 10 * v_get_line_height() + 6);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
                                        GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(listPanel, scroll, FALSE, FALSE, 0);



    /* create the list view */
    _listView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(_listStore));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("", renderer,
                                                      "text", 1, NULL);

    gtk_tree_view_append_column(GTK_TREE_VIEW(_listView), column);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(_listView), FALSE);

    _selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(_listView));
    g_signal_connect(G_OBJECT(_selection), "changed",
                     G_CALLBACK(_on_active_item_changed), NULL);

    gtk_container_add(GTK_CONTAINER(scroll), _listView);


    /* show the list view */
    gtk_widget_show(_listView);
    gtk_widget_show(scroll);



    /* create the "New" and "Remove" buttons */
    buttonBox = gtk_hbutton_box_new();
    gtk_box_pack_start(listPanel, buttonBox, FALSE, FALSE, 6);

    button = gtk_button_new_with_label(_T("New"));
    gtk_container_add(GTK_CONTAINER(buttonBox), button);
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(_on_new_matrix_clicked), NULL);
    gtk_widget_show(button);

    _removeButton = gtk_button_new_with_label(_T("Remove"));
    gtk_widget_set_sensitive(_removeButton, FALSE);
    gtk_container_add(GTK_CONTAINER(buttonBox), _removeButton);
    g_signal_connect(G_OBJECT(_removeButton), "clicked",
                     G_CALLBACK(_on_remove_matrix_clicked), NULL);
    gtk_widget_show(_removeButton);

    gtk_widget_show(buttonBox);


    /* add callback function to stay informed whenever the semiring changes */
    m_semirings_add_hook(OnActiveItemHasChanged, _on_semiring_changed);
}



/* Creates the right side of the dialog containing the completely editable
 * matrix and its size. */
static void _create_matrix_panel(GtkBox *matrixPanel)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkWidget *matrix;
    guint row, column, size;



    /*  create the size section */
    panel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(matrixPanel), panel, FALSE, FALSE, 0);

    label = gtk_label_new(_T("Matrix size:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    _matrixSizeEntry = gtk_spin_button_new_with_range(
                                      1, prop_get_max_matrix_size(), 1);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(_matrixSizeEntry),
                              prop_get_max_matrix_size());
    gtk_entry_set_editable(GTK_ENTRY(_matrixSizeEntry), FALSE);
    gtk_entry_set_width_chars(GTK_ENTRY(_matrixSizeEntry), 3);
    gtk_entry_set_max_length(GTK_ENTRY(_matrixSizeEntry), 3);

    g_signal_connect(G_OBJECT(_matrixSizeEntry), "value-changed",
                     G_CALLBACK(_on_matrix_size_changed), NULL);

    gtk_box_pack_start(GTK_BOX(panel), _matrixSizeEntry, FALSE, FALSE, 12);

    gtk_widget_show(_matrixSizeEntry);


    gtk_widget_show(panel);



    /*  create the matrix section */
    size = prop_get_max_matrix_size();
    matrix = gtk_table_new(size, size, TRUE);
    gtk_table_set_row_spacings(GTK_TABLE(matrix), 0);
    gtk_table_set_col_spacings(GTK_TABLE(matrix), 0);
    gtk_box_pack_start(GTK_BOX(matrixPanel), matrix, FALSE, FALSE, 12);

    _matrixItem = g_new0(GtkWidget**, size);
    for (row=0; row < size; row++)
    {
        _matrixItem[row] = g_new0(GtkWidget*, size);
        for (column=0; column < size; column++)
        {
            guint coord = row * size + column;

            _matrixItem[row][column] = gtk_entry_new();
            gtk_entry_set_width_chars(GTK_ENTRY(_matrixItem[row][column]), 5);
            gtk_table_attach_defaults(GTK_TABLE(matrix),
                                      _matrixItem[row][column],
                                      column, column+1, row, row+1);
            g_signal_connect(G_OBJECT(_matrixItem[row][column]),
                             "focus-out-event",
                             G_CALLBACK(_on_entry_lost_focus),
                             GUINT_TO_POINTER(coord));
            gtk_widget_show(_matrixItem[row][column]);
        }
    }

    gtk_widget_show(matrix);
}



/* Creates the main panel holding all other components of the dialog. */
static void _create_main_panel(GtkWidget *rootPanel)
{
    GtkWidget *mainPanel;
    GtkWidget *helpLabel;
    GtkWidget *listPanel;
    GtkWidget *matrixPanel;


    /* create a description what to do in this dialog */
    helpLabel = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(helpLabel),
                         _T("Please select a matrix from the list below or "
                            "press 'New' to create a new matrix."));
    gtk_misc_set_alignment(GTK_MISC(helpLabel), 0, 0);
    gtk_misc_set_padding(GTK_MISC(helpLabel), 6, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), helpLabel, FALSE, FALSE, 6);
    gtk_widget_show(helpLabel);


    /* create the main panel holding the list view and the matrix section */
    mainPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), mainPanel, FALSE, FALSE, 6);


    /* create the list view */
    listPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), listPanel, FALSE, FALSE, 6);

    _create_list(GTK_BOX(listPanel));

    gtk_widget_show(listPanel);


    /* create the matrix section */
    matrixPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), matrixPanel, FALSE, FALSE, 6);

    _create_matrix_panel(GTK_BOX(matrixPanel));

    gtk_widget_show(matrixPanel);


    gtk_widget_show(mainPanel);
}




/* Shows a dialog in which the user can choose a matrix from a list or add
 * a new matrix. All matrices can be edited and removed. */
void v_show_matrix_dialog(void)
{
    static GtkWidget *dialog = NULL;
    gint result;

    guint index;
    GtkTreePath *path;


    /* create the dialog only once */
    if (dialog == NULL)
    {
        dialog =  gtk_dialog_new_with_buttons(
                            _T("Select or enter a matrix"),
                            v_get_main_window(),
                            GTK_DIALOG_DESTROY_WITH_PARENT,
                            _T("OK"),
                            GTK_RESPONSE_ACCEPT,
                            _T("Cancel"),
                            GTK_RESPONSE_REJECT,
                            NULL);
        gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
        gtk_dialog_set_default_response(GTK_DIALOG(dialog),
                                        GTK_RESPONSE_ACCEPT);

        _create_main_panel(GTK_DIALOG(dialog)->vbox);
    }


    /* select the current matrix from the list */
    index = m_matrices_get_selected_index();
    if (index == G_MAXUINT) index = 0;
    path = gtk_tree_path_new_from_indices(index, -1);
    gtk_tree_selection_select_path(_selection, path);
    gtk_tree_path_free(path);

    /* if nothing is selected by now, update the matrix (it will be disabled) */
    if (v_get_selected_list_item_index(_selection))
    {
        _on_active_item_changed(_selection, NULL);
    }


    /* focus the list view */
    gtk_widget_grab_focus(_listView);


    /* show the dialog */
    result = gtk_dialog_run(GTK_DIALOG(dialog));


    /* select a matrix if the user pressed "OK" */
    if (result == GTK_RESPONSE_ACCEPT)
    {
        m_matrices_select(v_get_selected_list_item_index(_selection));
    }


    /* only hide the dialog to fasten the next show up of this dialog */
    gtk_widget_hide(dialog);
}
