/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "View.Private.h"
#include "Model.h"



/* the menu item with label "Select semiring" */
static GtkWidget *_changeSemiringItem;

/* the menu item with label "Select initial matrix" */
static GtkWidget *_changeMatrixItem;


/* the menu item with label "Compute result" */
static GtkWidget *_computeItem;

/* the menu item with label "Compute all steps" */
static GtkWidget *_computeStepwiseItem;




/* Called whenever the active algorithm has changed. */
static void _on_algorithm_changed(gpointer notUsed)
{
    /* if there is an active algorithm: */
    if (m_algorithms_get_selected() != NULL)
    {
        /* enable the "Select semiring" menu item */
        gtk_widget_set_sensitive(_changeSemiringItem, TRUE);
    }
    else    /* if there is no active algorithm */
    {
        /* disable both "Select semiring" and "Select initial matrix" item */
        gtk_widget_set_sensitive(_changeSemiringItem, FALSE);
        gtk_widget_set_sensitive(_changeMatrixItem, FALSE);
    }
}



/* Called whenever the active semiring has changed. */
static void _on_semiring_changed(gpointer notUsed)
{
    /* only the "Select initial matrix" menu item is influenced */
    gtk_widget_set_sensitive(_changeMatrixItem,
                             m_semirings_get_selected() != NULL);
}



/* Called whenever the active semiring has changed. */
static void _on_matrix_changed(gpointer notUsed)
{
    gboolean enable = m_matrices_get_selected() != NULL;

    /* enable or disable the "Compute XXX" menu items */
    gtk_widget_set_sensitive(_computeItem, enable);
    gtk_widget_set_sensitive(_computeStepwiseItem, enable);
}



/* Called whenever the active semiring has changed. */
static void _on_computation_finished(gpointer notUsed)
{
    if (m_computation_get_last() == OnlyResultComputed)
    {
        /* enable or disable the "Compute XXX" menu items */
        gtk_widget_set_sensitive(_computeItem, FALSE);
        gtk_widget_set_sensitive(_computeStepwiseItem, TRUE);
    }
    else if (m_computation_get_last() == AllStepsComputed)
    {
        /* enable or disable the "Compute XXX" menu items */
        gtk_widget_set_sensitive(_computeItem, TRUE);
        gtk_widget_set_sensitive(_computeStepwiseItem, FALSE);
    }
}



/* Types of signals used in this module.
 * Each type corresponds with a menu item and a function to call when the
 * user selected the menu item. */
enum MenuItemID
{
    ChangeAlgorithm,
    ChangeSemiring,
    ChangeMatrix,
    ComputeResult,
    ComputeStepwise,

    Quit,

    ToggleAlgorithmDescription,
    ToggleTooltips,

    About
};


/* Adapter for correctly handling and distributing the signals. */
static void _on_menu_item(GtkMenuItem *item, gpointer userData)
{
    switch ((enum MenuItemID) (GPOINTER_TO_UINT(userData)))
    {
        case ChangeAlgorithm: v_show_algorithm_dialog(); break;

        case ChangeSemiring:  v_show_semiring_dialog(); break;

        case ChangeMatrix:    v_show_matrix_dialog(); break;

        case ComputeResult:   v_compute(FALSE); break;

        case ComputeStepwise: v_compute(TRUE); break;

        case Quit:            gtk_main_quit(); break;

        case About:           v_show_about_dialog(); break;

        default: break;
    }
}


/* Adapter for correctly handling and distributing the signals. */
static void _on_check_menu_item(GtkCheckMenuItem *item, gpointer userData)
{
    gboolean active = gtk_check_menu_item_get_active(item);

    switch ((enum MenuItemID) (GPOINTER_TO_UINT(userData)))
    {
        case ToggleAlgorithmDescription:
            v_show_algorithm_description(active);
            break;

        case ToggleTooltips:
            v_show_tooltips(active);
            break;

        default: break;
    }
}




/* Creates the "Application" menu and its menu items. */
static GtkWidget* _create_application_menu(void)
{
    GtkWidget *menu;
    GtkWidget *menuItem;


    /* create the container for the submenu */
    menu = gtk_menu_new();


    /* create the menu items */

    menuItem = gtk_menu_item_new_with_mnemonic(_T("Select algorithm ..."));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(ChangeAlgorithm));
    gtk_widget_show(menuItem);


    menuItem = gtk_menu_item_new_with_mnemonic(_T("Select semiring ..."));
    gtk_widget_set_sensitive(menuItem, FALSE);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(ChangeSemiring));
    gtk_widget_show(menuItem);

    /* store a reference to the "Select semiring" item to enable it later on */
    _changeSemiringItem = menuItem;


    menuItem = gtk_menu_item_new_with_mnemonic(_T("Select initial matrix ..."));
    gtk_widget_set_sensitive(menuItem, FALSE);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(ChangeMatrix));
    gtk_widget_show(menuItem);

    /* store a reference to the "Select initial matrix" item to enable it
     * later on */
    _changeMatrixItem = menuItem;


    menuItem = gtk_separator_menu_item_new();
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    gtk_widget_show(menuItem);


    menuItem = gtk_menu_item_new_with_mnemonic(_T("Compute result"));
    gtk_widget_set_sensitive(menuItem, FALSE);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(ComputeResult));
    gtk_widget_show(menuItem);

    /* store a reference to the "Select initial matrix" item to enable it
     * later on */
     _computeItem = menuItem;


    menuItem = gtk_menu_item_new_with_mnemonic(_T("Compute all steps"));
    gtk_widget_set_sensitive(menuItem, FALSE);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(ComputeStepwise));
    gtk_widget_show(menuItem);

    /* store a reference to the "Select initial matrix" item to enable it
     * later on */
     _computeStepwiseItem = menuItem;


    menuItem = gtk_separator_menu_item_new();
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    gtk_widget_show(menuItem);


    menuItem = gtk_menu_item_new_with_mnemonic(_T("Quit"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item), GUINT_TO_POINTER(Quit));
    gtk_widget_show(menuItem);


    /* create the "Application" menu */
    menuItem = gtk_menu_item_new_with_mnemonic(_T("Application"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menu);
    gtk_widget_show(menuItem);



    /* add callback functions to stay informed whenever the algorithm,
     * the semiring or the matrix changes or a computation finishes */
    m_algorithms_add_hook(OnActiveItemHasChanged, _on_algorithm_changed);
    m_semirings_add_hook(OnActiveItemHasChanged, _on_semiring_changed);
    m_matrices_add_hook(OnActiveItemHasChanged, _on_matrix_changed);
    m_computation_add_hook(OnComputationFinished, _on_computation_finished);


    return menuItem;
}




/* Creates the "View" menu and its menu items. */
static GtkWidget* _create_view_menu(void)
{
    GtkWidget *menu;
    GtkWidget *menuItem;
    const gchar *string;


    /* create the container for the submenu */
    menu = gtk_menu_new();


    /* create the menu items */

    string = _T("Show algorithm description");
    menuItem = gtk_check_menu_item_new_with_mnemonic(string);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "toggled",
                     G_CALLBACK(_on_check_menu_item),
                     GUINT_TO_POINTER(ToggleAlgorithmDescription));
    gtk_widget_show(menuItem);


    menuItem = gtk_check_menu_item_new_with_mnemonic(_T("Show tooltips"));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItem), TRUE);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "toggled",
                     G_CALLBACK(_on_check_menu_item),
                     GUINT_TO_POINTER(ToggleTooltips));
    gtk_widget_show(menuItem);


    /* create the "View" menu */
    menuItem = gtk_menu_item_new_with_mnemonic(_T("View"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menu);
    gtk_widget_show(menuItem);

    return menuItem;
}



/* Creates the "Help" menu. */
static GtkWidget* _create_help_menu(void)
{
    GtkWidget *menu;
    GtkWidget *menuItem;


    /* create the container for the submenu */
    menu = gtk_menu_new();


    /* create the menu items */

    menuItem = gtk_menu_item_new_with_mnemonic(_T("About AppC"));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem);
    g_signal_connect(G_OBJECT(menuItem), "activate",
                     G_CALLBACK(_on_menu_item),
                     GUINT_TO_POINTER(About));
    gtk_widget_show(menuItem);


    /* create the "Help" menu */
    menuItem = gtk_menu_item_new_with_mnemonic(_T("Help"));
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menu);
    gtk_widget_show(menuItem);

    return menuItem;
}



/* Creates the menu. */
void v_init_menu(GtkBox *mainPanel)
{
    GtkWidget *menuBar = gtk_menu_bar_new();
    gtk_box_pack_start(mainPanel, menuBar, FALSE, FALSE, 0);

    /* create the submenus and add them to the menubar */
    gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), _create_application_menu());
    gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), _create_view_menu());
    gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), _create_help_menu());

    gtk_widget_show(menuBar);
}
