/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* the list view containing all available semirings */
static GtkWidget *_listView;

/* holds all selectable semirings (used by the list view) */
static GtkListStore *_listStore = NULL;

/* describes the selected items of the list view */
static GtkTreeSelection *_selection;



/* holds the description of the currently selected semiring */
static GtkTextBuffer *_descriptionBuffer;




/* Called whenever the currently selected algorithm has changed. */
static void _on_algorithm_changed(gpointer notUsed)
{
    GtkTreeIter iter;
    guint i;

    /* clear the semirings list */
    gtk_list_store_clear(_listStore);

    /* and refill it with the currently possible semirings */
    for (i=0; i < m_semirings_length(); i++)
    {
        Semiring semiring = m_semirings_get(i);

        gtk_list_store_append(_listStore, &iter);
        gtk_list_store_set(_listStore, &iter, 0, i, 1, semiring->name, -1);
    }
}



/* Called whenever the selected item in the list view has changed. */
static void _on_active_item_changed(GtkTreeSelection *selection,                                                     gpointer notUsed)
{
    guint selected = v_get_selected_list_item_index(selection);
    Semiring semiring = m_semirings_get(selected);

    /* update the description */
    gtk_text_buffer_set_text(_descriptionBuffer,
                             (semiring == NULL)
                                ? _T("No semiring description available.")
                                : semiring->description,
                             -1);
}



/* Creates the list view holding all selectable semirings. */
static void _create_list(GtkBox *listPanel)
{
    GtkWidget *label;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    GtkTreeIter iter;
    guint i;

    GtkWidget *scroll;



    /* create label for the list view */
    label = gtk_label_new(_T("Available semirings:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(listPanel, label, FALSE, FALSE, 6);
    gtk_widget_show(label);



    /* collect all semirings */
    _listStore = gtk_list_store_new(2, G_TYPE_UINT, G_TYPE_STRING);
    for (i=0; i < m_semirings_length(); i++)
    {
        Semiring semiring = m_semirings_get(i);

        gtk_list_store_append(_listStore, &iter);
        gtk_list_store_set(_listStore, &iter, 0, i, 1, semiring->name, -1);
    }


    /* create the scroll window */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_widget_set_size_request(scroll, 200, 13 * v_get_line_height() + 6);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
                                        GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(listPanel, scroll, FALSE, FALSE, 0);



    /* create the list view */
    _listView = gtk_tree_view_new_with_model(GTK_TREE_MODEL(_listStore));

    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes("", renderer,
                                                      "text", 1, NULL);

    gtk_tree_view_append_column(GTK_TREE_VIEW(_listView), column);
    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(_listView), FALSE);

    _selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(_listView));
    g_signal_connect(G_OBJECT(_selection), "changed",
                     G_CALLBACK(_on_active_item_changed), NULL);

    gtk_container_add(GTK_CONTAINER(scroll), _listView);


    /* show the list view */
    gtk_widget_show(_listView);
    gtk_widget_show(scroll);


    /* add callback function to stay informed whenever the algorithm changes */
    m_algorithms_add_hook(OnActiveItemHasChanged, _on_algorithm_changed);
}



/* Creates the right side of the dialog containing the description text field.
 */
static void _create_info(GtkBox *infoPanel)
{
    GtkWidget *panel;
    GtkWidget *label;
    GtkTextView *view;


    /*  create the description section */
    panel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(infoPanel), panel, FALSE, FALSE, 0);

    label = gtk_label_new(_T("Semiring description:"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(panel), label, FALSE, FALSE, 6);
    gtk_widget_show(label);

    view = v_create_text_field(GTK_BOX(panel), 300, 13);
    _descriptionBuffer = gtk_text_view_get_buffer(view);
    gtk_text_buffer_set_text(_descriptionBuffer,
                             _T("No semiring description available."), -1);

    gtk_widget_show(panel);
}



/* Creates the main panel holding all other components of the dialog. */
static void _create_main_panel(GtkWidget *rootPanel)
{
    GtkWidget *mainPanel;
    GtkWidget *helpLabel;
    GtkWidget *listPanel;
    GtkWidget *infoPanel;


    /* create a description what to do in this dialog */
    helpLabel = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(helpLabel),
                         _T("Please select a semiring from the list below."));
    gtk_misc_set_alignment(GTK_MISC(helpLabel), 0, 0);
    gtk_misc_set_padding(GTK_MISC(helpLabel), 6, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), helpLabel, FALSE, FALSE, 6);
    gtk_widget_show(helpLabel);


    /* create the main panel holding the list view and the info section */
    mainPanel = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(rootPanel), mainPanel, FALSE, FALSE, 6);


    /* create the list view */
    listPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), listPanel, FALSE, FALSE, 6);

    _create_list(GTK_BOX(listPanel));

    gtk_widget_show(listPanel);


    /* create the info section */
    infoPanel = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainPanel), infoPanel, FALSE, FALSE, 6);

    _create_info(GTK_BOX(infoPanel));

    gtk_widget_show(infoPanel);


    gtk_widget_show(mainPanel);
}




/* Creates a warning dialog. */
static void _show_warning_dialog(void)
{
    GtkWidget *dialog;
    gchar *string;


    /* construct the warning text */
    string = g_strconcat(_T("Selecting a semiring is not possible because "
                            "either there is no matching semiring for the "
                            "selected algorithm or there are no semirings "
                            "in the semirings' directory."),
                         "\n\n",
                         _T("Please make sure there is at least one "
                            "semiring file in the semirings' "
                            "directory which fulfills the selected algorithm's "
                            "prerequisites."),
                         "\n",
                         _T("Note that new semirings are only available "
                            "after a restart of this application."),
                         NULL);


    /* create the dialog */
    dialog = gtk_message_dialog_new(
                        v_get_main_window(),
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_MESSAGE_WARNING,
                        GTK_BUTTONS_NONE,
                        string);
    g_free(string);

    gtk_window_set_title(GTK_WINDOW(dialog),
                         _T("No matching semiring was found"));
    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
                           _T("OK"), GTK_RESPONSE_ACCEPT,
                           NULL);
    gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);


    /* show the dialog and destroy it after the user pressed "OK" */
    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
}



/* Shows a dialog in which the user can choose a semiring from a list. */
void v_show_semiring_dialog(void)
{
    static GtkWidget *dialog = NULL;
    gint result;


    /* if there is at least one semiring, create the normal dialog */
    if (m_semirings_length() > 0)
    {
        /* create the dialog only once */
        if (dialog == NULL)
        {
            guint index;
            GtkTreePath *path;

            dialog =  gtk_dialog_new_with_buttons(
                                _T("Select a semiring"),
                                v_get_main_window(),
                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                _T("OK"),
                                GTK_RESPONSE_ACCEPT,
                                _T("Cancel"),
                                GTK_RESPONSE_REJECT,
                                NULL);
            gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
            gtk_dialog_set_default_response(GTK_DIALOG(dialog),
                                            GTK_RESPONSE_ACCEPT);

            /* create the dialog contents */
            _create_main_panel(GTK_DIALOG(dialog)->vbox);

            /* select the current semiring from the list */
            index = m_semirings_get_selected_index();
            if (index == G_MAXUINT) index = 0;
            path = gtk_tree_path_new_from_indices(index, -1);
            gtk_tree_selection_select_path(_selection, path);
            gtk_tree_path_free(path);
        }
        else    /* if the dialog is already created: */
        {
            /* focus the list view */
            gtk_widget_grab_focus(_listView);
        }
    }
    else    /* show warning dialog if there is no semiring */
    {
        _show_warning_dialog();
        return;
    }


    /* show the dialog */
    result = gtk_dialog_run(GTK_DIALOG(dialog));


    /* select a semiring if the user pressed "OK" */
    if (result == GTK_RESPONSE_ACCEPT)
    {
        m_semirings_select(v_get_selected_list_item_index(_selection));
    }


    /* only hide the dialog to fasten the next show up of this dialog */
    gtk_widget_hide(dialog);
}
