/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Model.h"
#include "View.Private.h"



/* the height of a text line using the standard font */
static guint _lineHeight;

/* list of all (helpful) tooltips */
static GtkTooltips *_tooltips;



/* Initializes the utilities module. */
void v_utils_initialize(void)
{
    /* compute the line height */
    GtkWidget *widget;
    PangoLayout *layout;
    int height;

    widget = gtk_label_new("");
    layout = gtk_widget_create_pango_layout(widget, "Some funny text.");
    pango_layout_get_pixel_size(layout, NULL, &height);
    g_object_unref(layout);
    gtk_widget_destroy(widget);

    _lineHeight = height;


    /* create the tooltips group */
    _tooltips = gtk_tooltips_new();
}




/* Returns the global text line height (for the standard font). */
guint v_get_line_height(void)
{
    return _lineHeight;
}



/* Sets the font family and the wrap mode of 'textview'.
 * Pass TRUE if a monospaced font and no wrap should be used in 'textview'.
 * Otherwise a sans serif font and word wrap is set. */
void v_set_monospace_font(GtkTextView *textview, gboolean monospace)
{
    PangoFontDescription *fontDesc;
    const gchar *family = (monospace) ? "Monospace" : "Sans";
    GtkWrapMode wrap = (monospace) ? GTK_WRAP_NONE : GTK_WRAP_WORD;

    fontDesc = pango_font_description_from_string(family);
    gtk_widget_modify_font(GTK_WIDGET(textview), fontDesc);
    pango_font_description_free(fontDesc);

    gtk_text_view_set_wrap_mode(textview, wrap);
}



/* Creates a static text field in 'panel' with 'lines' lines and a width of
 * 'width' pixels. */
GtkTextView *v_create_text_field(GtkBox *panel, guint width, guint lines)
{
    GtkWidget *view;
    GtkWidget *scroll;


    /* create and configure the scroll window */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_widget_set_size_request(scroll, width, lines * _lineHeight + 6);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
                                        GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(panel, scroll, FALSE, FALSE, 0);


    /* create and configure the view */
    view = gtk_text_view_new();
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
    gtk_widget_set_sensitive(view, FALSE);
    gtk_container_add(GTK_CONTAINER(scroll), view);


    gtk_widget_show(view);
    gtk_widget_show(scroll);


    return GTK_TEXT_VIEW(view);
}



/* Sets the tooltip 'tip' for 'widget'. */
void v_set_tooltip(GtkWidget *widget, const gchar *tip)
{
    gtk_tooltips_set_tip(_tooltips, widget, tip, tip);
}


/* Enables or disables all tooltips set by 'v_set_tooltip()'. */
void v_show_tooltips(gboolean show)
{
    if (show)
    {
        gtk_tooltips_enable(_tooltips);
    }
    else
    {
        gtk_tooltips_disable(_tooltips);
    }
}



/* Returns the first index of selected items found in 'selection'. */
guint v_get_selected_list_item_index(GtkTreeSelection *selection)
{
    guint result = G_MAXUINT;
    GtkTreeIter iter;
    GtkTreeModel *model;

    if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
        gtk_tree_model_get(model, &iter, 0, &result, -1);
    }

    return result;
}

