/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/* Defines the interface of an algorithm.
 * Each algorithm must implement this interface. Additionally, there must be
 * an exported function of type AlgorithmInitializer in each algorithm file.
 * See below for more information. */

#ifndef ALGORITHM_H
#define ALGORITHM_H



/* Include needed files. */
#include "AppC.h"
#include "Semiring.h"
#include "Matrix.h"



/* The algorithm interface. Each algorithm must implement all of these
 * functions. The fields are initialized by the application's loader and
 * must not be altered or freed by the algorithm module. */
typedef struct AlgorithmStruct *Algorithm;
struct AlgorithmStruct
{
    /* The name of the algorithm file without ending. */
    gchar *filename;

    /* The full name of this algorithm. */
    gchar *name;

    /* A description of this algorithm. This string may contain markup data. */
    gchar *description;

    /* A list of semiring names acceptable for this algorithm. */
    gchar **accepted_semirings;

    /* Pseudocode of this algorithm to describe what it is exactly doing.
     * This string may contain markup data. */
    gchar *pseudocode;


    /* The dynamically loaded module containing this algorithm. */
    GModule *module;




    /* Executes the algorithm.
     *
     * initial : Matrix needed as input by the algorithm. The algorithm must
     *           not alter this matrix.
     * result : Matrix to store the computation's result in.
     */
    void (*compute)(Matrix result, const Matrix initial);



    /* Prepares the algorithm for stepwise execution. Sets the initial (weight)
     * matrix as starting point of the algorithm's computation.
     *
     * m : The initial (weight) matrix for the algorithm. The algorithm must
     *     not alter this matrix.
     */
    void (*prepare)(const Matrix initial);



    /* Executes one step of the algorithm.
     * The function 'prepare' must be called before the first call to this
     * function.
     * In the case of FALSE as return value, this function must only be called
     * again after setting a new initial matrix via 'prepare'.
     *
     * result : Matrix to store the computation's result in.
     *
     * Returns: TRUE if there is a next step after the one just computed,
     *          otherwise FALSE.
     */
    gboolean (*compute_next_step)(Matrix result);
};



/* Each algorithm must define the function "algorithm_initialize" as follows:
 *
 *    G_MODULE_EXPORT void algorithm_initialize(Algorithm alg,
 *                                              const MatrixFunctions *mf)
 *    {
 *      ...
 *    }
 *
 * It is called by the loader and should fill in the structure 'alg' passed as
 * argument.
 *
 * alg : A pointer to an AlgorithmStruct which should be filled in by the
 *       algorithm.
 *
 * mf : A collection of useful matrix functions. See Matrix.h for the
 *      definition of 'MatrixFunctions'.
 */
typedef void (*AlgorithmInitializer)(Algorithm alg, const MatrixFunctions *mf);


#endif /* ALGORITHM_H */
