/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/* Defines the interface of a semiring.
 * Each semiring must implement this interface. Additionally, there must be
 * an exported function of type SemiringInitializer in each semiring file.
 * See below for more information. */

#ifndef SEMIRING_H
#define SEMIRING_H



/* Include needed files. */
#include "AppC.h"




/* An opaque data type for semiring items.
 * Do not modify SemiringItems directly, do so only with the help of the
 * appropriate semiring operations. */
typedef gpointer SemiringItem;



/* The semiring interface. Each semiring must implement all of these
 * functions. The fields are initialized by the application's loader and
 * must not be altered or freed by the semiring module. */
typedef struct SemiringStruct *Semiring;
struct SemiringStruct
{
    /* The name of the semiring file without ending. */
    gchar *filename;

    /* The full name of this semiring. */
    gchar *name;

    /* A description of this semiring. This string may contain markup data. */
    gchar *description;


    /* The dynamically loaded module containing this semiring. */
    GModule *module;





    /* Returns the neutral element of this semiring regarding its addition.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*zero)(void);

    /* Returns the neutral element of this semiring regarding its
     * multiplication.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*one)(void);

    /* Returns the result of the star operation applied to a.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*star)(SemiringItem a);

    /* Returns the sum of a and b.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*add)(SemiringItem a, SemiringItem b);

    /* Returns the product of a and b.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*mult)(SemiringItem a, SemiringItem b);



    /* Returns TRUE if a and be are equal, otherwise FALSE. */
    gboolean (*equals)(SemiringItem a, SemiringItem b);


    /* Creates a copy of 'item'.
     * Pass the returned  object to 'release()' when it is not used anymore. */
    SemiringItem (*copy)(SemiringItem item);




    /* Scans a semiring item from 'string'.
     * If 'read' is not NULL it should hold the number of read characters when
     * this function returns.
     * Pass the returned  object to 'release()' when it is not used anymore.
     *
     * string : The buffer which begins with the string representation of an
     *          semiring item.
     * read : Place to store the length of the semiring item's string
     *        representation.
     *
     * Returns : The extracted semiring item or Zero() on error.
     */
    SemiringItem (*from_string)(const gchar *string, guint *read);

    /* Prints a semiring item into a newly allocated buffer.
     * The function 'g_strdup_printf()' may be used for this purpose.
     * Call 'g_free()' on the result if it is no longer used.
     *
     * item : The semiring item to print.
     *
     * Returns : A newly allocated string holding the string representation
     *           of 'item'.
     */
    gchar* (*to_string)(SemiringItem item);



    /* Releases a semiring item. This function must be called for every
     * semiring item which is not be used anymore. */
    void (*release)(SemiringItem item);
};



/* Each semiring must define the function "semiring_initialize" as follows:
 *
 *    G_MODULE_EXPORT void semiring_initialize(Semiring sr)
 *    {
 *      ...
 *    }
 *
 * It is called by the loader and should fill in the structure passed as
 * argument.
 *
 * sr : A pointer to a SemiringStruct which should be filled in by the
 *      semiring.
 */
typedef void (*SemiringInitializer)(Semiring sr);


#endif /* SEMIRING_H */
