/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* A module used to provide the field of fractional numbers. */
#ifndef FRACTION_H
#define FRACTION_H


/* Include needed files. */
#include "AppC.h"





/* Definition of the sign type of fractions. */
typedef enum { Plus, Minus } Sign;


/* Definition of the fraction type. */
typedef struct FractionStruct
{
    Sign sign;
    guint numerator;
    guint denominator;
} *Fraction;




/* Returns the fraction representing the value zero. */
Fraction fraction_zero(void);

/* Returns the fraction representing the value one. */
Fraction fraction_one(void);



/* Returns the absolute fraction of 'a', e.g. if 'a' is negative '-a' is
 * returned, otherwise a copy of 'a'. */
Fraction fraction_abs(Fraction a);

/* Returns a fraction with changed sign compared to 'a'. */
Fraction fraction_negate(Fraction a);

/* Returns the inverse of 'a', e.g. '1/a' is returned. */
Fraction fraction_inverse(Fraction a);

/* Returns the canceled fraction of 'a'. */
Fraction fraction_cancel(Fraction a);




/* Returns the canceled sum of 'a' and 'b'. */
Fraction fraction_add(Fraction a, Fraction b);

/* Returns the canceled difference of 'a' and 'b'. */
Fraction fraction_sub(Fraction a, Fraction b);

/* Returns the canceled product of 'a' and 'b'. */
Fraction fraction_mul(Fraction a, Fraction b);

/* Returns the canceled quotient of 'a' and 'b'. */
Fraction fraction_div(Fraction a, Fraction b);



/* Returns TRUE if 'a' and 'b' are equal, otherwise FALSE. */
gboolean fraction_equals(Fraction a, Fraction b);


/* Creates a copy of 'a'. */
Fraction fraction_copy(Fraction a);


/* Returns the minimum of 'a' and 'b'. */
Fraction fraction_min(Fraction a, Fraction b);

/* Returns the maximum of 'a' and 'b'. */
Fraction fraction_max(Fraction a, Fraction b);



/* Scans a fraction from 'string'. if 'read' is not NULL, it will be used to
 * store the length of the scanned fraction.
 * Returns the scanned fraction or 'zero()' on error. */
Fraction fraction_from_string(const gchar *string, guint *read);

/* Returns a string representation of 'a'. */
gchar* fraction_to_string(Fraction a);



/* Creates a new fraction as a copy of 'a'. */
Fraction fraction_copy(Fraction a);


/* Frees the unused fraction 'a'. */
void fraction_free(Fraction a);


#endif /* FRACTION_H */
