/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"
#include "Fraction.h"




typedef struct ArcticalStruct
{
    gboolean minusInfinity;
    Fraction value;
} *Arctical;




static SemiringItem copy(SemiringItem item)
{
    Arctical a = (Arctical) item;
    Arctical t = g_new0(struct ArcticalStruct, 1);

    t->minusInfinity = a->minusInfinity;
    t->value = fraction_copy(a->value);

    return (SemiringItem) t;
}



static void release(SemiringItem item)
{
    Arctical a = (Arctical) item;

    fraction_free(a->value);
    g_free(a);
}


static SemiringItem zero(void)
{
    Arctical z = g_new0(struct ArcticalStruct, 1);

    z->minusInfinity = TRUE;
    z->value = fraction_zero();

    return (SemiringItem) z;
}


static SemiringItem one(void)
{
    Arctical o = g_new0(struct ArcticalStruct, 1);

    o->minusInfinity = FALSE;
    o->value = fraction_zero();

    return (SemiringItem) o;
}


static SemiringItem star(SemiringItem a)
{
    return one();
}


static SemiringItem add(SemiringItem a, SemiringItem b)
{
    Arctical _a = (Arctical) a;
    Arctical _b = (Arctical) b;


    if (_a->minusInfinity == TRUE)
    {
        return copy(b);
    }
    else if (_b->minusInfinity == TRUE)
    {
        return copy(a);
    }
    else
    {
        Arctical t = g_new0(struct ArcticalStruct, 1);

        t->minusInfinity = FALSE;
        t->value = fraction_min(_a->value, _b->value);

        return t;
    }
}


static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    Arctical _a = (Arctical) a;
    Arctical _b = (Arctical) b;

    if (_a->minusInfinity == TRUE || _b->minusInfinity == TRUE)
    {
        return zero();
    }
    else
    {
        Arctical t = g_new0(struct ArcticalStruct, 1);

        t->minusInfinity = FALSE;
        t->value = fraction_add(_a->value, _b->value);

        return t;
    }
}



static gboolean equals(SemiringItem a, SemiringItem b)
{
    Arctical _a = (Arctical) a;
    Arctical _b = (Arctical) b;

    if (_a->minusInfinity != _b->minusInfinity)
    {
        return FALSE;
    }
    else if (_a->minusInfinity == TRUE)  /* && _b->minusInfinity == TRUE */
    {
        return TRUE;
    }
    else
    {
        return fraction_equals(_a->value, _b->value);
    }
}


static SemiringItem from_string(const gchar *string, guint *read)
{
    if (!strcmp(string, "-oo"))
    {
        if (read != NULL) *read = 3;
        return zero();
    }
    else
    {
        Arctical t = g_new0(struct ArcticalStruct, 1);

        t->minusInfinity = FALSE;
        t->value = fraction_from_string(string, read);

        /* accept only positive numbers */
        if (t->value->sign == Plus || t->value->numerator == 0)
        {
            return t;
        }
        else
        {
            release(t);

            if (read != NULL) *read = 0;
            return zero();
        }
    }
}


static char* to_string(SemiringItem item)
{
    Arctical t = (Arctical) item;

    if (t->minusInfinity == TRUE)
    {
        return g_strdup("-oo");
    }
    else
    {
        return fraction_to_string(t->value);
    }
}



G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
