/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "Fraction.h"




/* Returns the greatest common divisor of 'a' and 'b' (using the Euclidean
 * algorithm). */
static guint _gcd(guint a, guint b)
{
    unsigned int h;

    if (a < b) { h = a; a = b; b = h; }

    while (b != 0)
    {
        h = a % b;
        a = b;
        b = h;
    }

    return a;
}

/* Returns the least common multiple of 'a' and 'b'. */
static guint _lcm(guint a, guint b)
{
    return a * (b / _gcd(a, b));
}





/* Returns the fraction representing the value zero. */
Fraction fraction_zero(void)
{
    Fraction z = g_new0(struct FractionStruct, 1);

    z->sign = Plus;
    z->numerator = 0;
    z->denominator = 1;

    return z;
}


/* Returns the fraction representing the value one. */
Fraction fraction_one(void)
{
    Fraction o = g_new0(struct FractionStruct, 1);

    o->sign = Plus;
    o->numerator = 1;
    o->denominator = 1;

    return o;
}



/* Returns the absolute fraction of 'a', e.g. if 'a' is negative '-a' is
 * returned, otherwise a copy of 'a'. */
Fraction fraction_abs(Fraction a)
{
    Fraction b = g_new0(struct FractionStruct, 1);

    b->sign = Plus;
    b->numerator = a->numerator;
    b->denominator = a->denominator;

    return b;
}

/* Returns a fraction with changed sign compared to 'a'. */
Fraction fraction_negate(Fraction a)
{
    Fraction b = g_new0(struct FractionStruct, 1);

    if (a->numerator != 0)
    {
        b->sign = (a->sign == Plus) ? Minus : Plus;
    }
    b->numerator = a->numerator;
    b->denominator = a->denominator;

    return b;
}

/* Returns the inverse of 'a', e.g. '1/a' is returned. */
Fraction fraction_inverse(Fraction a)
{
    Fraction b = g_new0(struct FractionStruct, 1);

    b->sign = a->sign;

    /* exchange numerator and denumerator */
    b->numerator = a->denominator;
    b->denominator = a->numerator;

    return b;
}

/* Returns the canceled fraction of 'a'. */
Fraction fraction_cancel(Fraction a)
{
    Fraction b = g_new0(struct FractionStruct, 1);
    unsigned int gcd = _gcd(a->numerator, a->denominator);

    b->sign = a->sign;

    /* cancel down */
    b->numerator = a->numerator / gcd;
    b->denominator = a->denominator / gcd;

    return b;
}


/* Returns the canceled sum of 'a' and 'b'. */
Fraction fraction_add(Fraction a, Fraction b)
{
    Fraction c = g_new0(struct FractionStruct, 1);
    Fraction d;
    unsigned int lcm = _lcm(a->denominator, b->denominator);

    unsigned int _a = (a->numerator * (lcm / a->denominator));
    unsigned int _b = (b->numerator * (lcm / b->denominator));

    /* sum up */
    if (a->sign == b->sign)
    {
        c->numerator = _a + _b;
        c->sign = a->sign;
    }
    else    /* different signs */
    {
        if (_a > _b)
        {
            c->numerator = _a - _b;
            c->sign = a->sign;
        }
        else if (_b > _a)
        {
            c->numerator = _b - _a;
            c->sign = b->sign;
        }
        else
        {
            c->numerator = 0;
            c->sign = Plus;
        }
    }
    c->denominator = lcm;

    /* cancel down the result */
    d = fraction_cancel(c);
    fraction_free(c);

    return d;
}

/* Returns the canceled difference of 'a' and 'b'. */
Fraction fraction_sub(Fraction a, Fraction b)
{
    Fraction neg_b = fraction_negate(b);
    Fraction result = fraction_add(a, neg_b);
    fraction_free(neg_b);
    return result;
}

/* Returns the canceled product of 'a' and 'b'. */
Fraction fraction_mul(Fraction a, Fraction b)
{
    Fraction c = g_new0(struct FractionStruct, 1);
    Fraction d;

    c->numerator = a->numerator * b->numerator;
    c->denominator = a->denominator * b->denominator;
    c->sign = (a->sign == b->sign) ? Plus : Minus;

    /* cancel down the result */
    d = fraction_cancel(c);
    fraction_free(c);

    return d;
}

/* Returns the canceled quotient of 'a' and 'b'. */
Fraction fraction_div(Fraction a, Fraction b)
{
    Fraction inv_b = fraction_inverse(b);
    Fraction result = fraction_mul(a, inv_b);
    fraction_free(inv_b);
    return result;
}




/* Returns TRUE if 'a' and 'b' are equal, otherwise FALSE. */
gboolean fraction_equals(Fraction a, Fraction b)
{
    if (a->sign != b->sign)
    {
        return FALSE;
    }
    else
    {
        Fraction diff = fraction_sub(a, b);
        gboolean result = (diff->numerator == 0);
        fraction_free(diff);
        return result;
    }
}


/* Creates a copy of 'a'. */
Fraction fraction_copy(Fraction a)
{
    Fraction f = g_new0(struct FractionStruct, 1);

    f->sign = a->sign;
    f->numerator = a->numerator;
    f->denominator = a->denominator;

    return f;
}



/* Returns the minimum of 'a' and 'b'. */
Fraction fraction_min(Fraction a, Fraction b)
{
    Fraction diff = fraction_sub(a, b);

    if (diff->sign == Minus)  /* a < b */
    {
        fraction_free(diff);
        return fraction_copy(a);
    }
    else                     /* a >= b */
    {
        fraction_free(diff);
        return fraction_copy(b);
    }
}

/* Returns the maximum of 'a' and 'b'. */
Fraction fraction_max(Fraction a, Fraction b)
{
    Fraction diff = fraction_sub(a, b);

    if (diff->sign == Plus)  /* a >= b */
    {
        fraction_free(diff);
        return fraction_copy(a);
    }
    else                     /* a < b */
    {
        fraction_free(diff);
        return fraction_copy(b);
    }
}




/* Scans a fraction from 'string'. if 'read' is not NULL, it will be used to
 * store the length of the scanned fraction.
 * Returns the scanned fraction or 'zero()' on error. */
Fraction fraction_from_string(const gchar *string, guint *read)
{
    const gchar *p = string;
    Fraction a = fraction_zero();

    /* scan the sign */
    a->sign = Plus;
    if (*p == '-')
    {
        a->sign = Minus;
        p++;
    }
    else if (*p == '+')
    {
        p++;
    }

    /* scan the numerator */
    if (!sscanf(p, "%i", &(a->numerator)))
    {
        fraction_free(a);
        return fraction_zero();
    }
    p += strspn(p, "0123456789");

    /* scan denominator */
    if (*p == '/' && sscanf(++p, "%i", &(a->denominator)))
    {
        p += strspn(p, "0123456789");
    }


    if (read != NULL) *read = p - string;

    return a;
}


/* Returns a string representation of 'a'. */
gchar* fraction_to_string(Fraction a)
{
    if (a->denominator != 1)
    {
        return g_strdup_printf("%s%u/%u",
                               (a->sign == Minus) ? "-" : "",
                               a->numerator,
                               a->denominator);
    }
    else
    {
        return g_strdup_printf("%s%u",
                               (a->sign == Minus) ? "-" : "",
                               a->numerator);
    }
}



/* Frees the unused fraction 'a'. */
void fraction_free(Fraction a)
{
    g_free(a);
}
