/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"
#include "Fraction.h"




typedef struct InvertStruct
{
    gboolean undefined;
    Fraction value;
} *Invert;





static SemiringItem copy(SemiringItem item)
{
    Invert a = (Invert) item;
    Invert t = g_new0(struct InvertStruct, 1);

    t->undefined = a->undefined;
    t->value = fraction_copy(a->value);

    return (SemiringItem) t;
}



static void release(SemiringItem item)
{
    Invert a = (Invert) item;

    fraction_free(a->value);
    g_free(a);
}


static SemiringItem zero(void)
{
    Invert z = g_new0(struct InvertStruct, 1);

    z->undefined = FALSE;
    z->value = fraction_zero();

    return (SemiringItem) z;
}


static SemiringItem one(void)
{
    Invert o = g_new0(struct InvertStruct, 1);

    o->undefined = FALSE;
    o->value = fraction_one();

    return (SemiringItem) o;
}


static SemiringItem star(SemiringItem a)
{
    Invert t = (Invert) a;
    Fraction f_one = fraction_one();

    if (t->undefined)
    {
        fraction_free(f_one);
        return copy(t);
    }
    else if (fraction_equals(t->value, f_one))
    {
        Invert t = g_new0(struct InvertStruct, 1);

        t->undefined = TRUE;
        t->value = fraction_zero();

        fraction_free(f_one);
        return one();
    }
    else
    {
        Invert n = g_new0(struct InvertStruct, 1);
        Fraction diff = fraction_sub(f_one, t->value);

        n->undefined = FALSE;
        n->value = fraction_div(f_one, diff);

        fraction_free(f_one);
        fraction_free(diff);

        return n;
    }
}


static SemiringItem add(SemiringItem a, SemiringItem b)
{
    Invert _a = (Invert) a;
    Invert _b = (Invert) b;

    if (_a->undefined == TRUE)
    {
        return copy(a);
    }
    else if (_b->undefined == TRUE)
    {
        return copy(b);
    }
    else
    {
        Invert t = g_new0(struct InvertStruct, 1);

        t->undefined = FALSE;
        t->value = fraction_add(_a->value, _b->value);

        return t;
    }
}


static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    Invert _a = (Invert) a;
    Invert _b = (Invert) b;

    if (_a->undefined == TRUE)
    {
        return copy(a);
    }
    else if (_b->undefined == TRUE)
    {
        return copy(b);
    }
    else
    {
        Invert t = g_new0(struct InvertStruct, 1);

        t->undefined = FALSE;
        t->value = fraction_mul(_a->value, _b->value);

        return t;
    }
}



static gboolean equals(SemiringItem a, SemiringItem b)
{
    Invert _a = (Invert) a;
    Invert _b = (Invert) b;

    if (_a->undefined != _b->undefined)
    {
        return FALSE;
    }
    else if (_a->undefined == TRUE)  /* && _b->undefined == TRUE */
    {
        return TRUE;
    }
    else
    {
        return fraction_equals(_a->value, _b->value);
    }
}


static SemiringItem from_string(const gchar *string, guint *read)
{
    if (!strcmp(string, "u"))
    {
        Invert t = g_new0(struct InvertStruct, 1);

        t->undefined = TRUE;
        t->value = fraction_zero();

        if (read != NULL) *read = 1;
        return t;
    }
    else
    {
        Invert t = g_new0(struct InvertStruct, 1);

        t->undefined = FALSE;
        t->value = fraction_from_string(string, read);

        return t;
    }
}


static char* to_string(SemiringItem item)
{
    Invert t = (Invert) item;

    if (t->undefined == TRUE)
    {
        return g_strdup("u");
    }
    else
    {
        return fraction_to_string(t->value);
    }
}




G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
