/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"
#include "Fraction.h"




typedef struct NodeStruct *Node;
struct NodeStruct
{
    gchar label;
    Node left;
    Node right;
};





static SemiringItem copy(SemiringItem item)
{
    Node t = (Node) item;
    Node n;

    if (t == NULL) return NULL;

    n = g_new0(struct NodeStruct, 1);
    n->label = t->label;
    n->left = copy(t->left);
    n->right = copy(t->right);

    return (SemiringItem) n;
}



static void release(SemiringItem item)
{
    Node n = (Node) item;

    if (n != NULL)
    {
        release(n->left);
        release(n->right);
        g_free(n);
    }
}


static SemiringItem zero(void)
{
    Node n = g_new0(struct NodeStruct, 1);

    n->label = 'O';
    n->left = n->right = NULL;

    return (SemiringItem) n;
}


static SemiringItem one(void)
{
    Node n = g_new0(struct NodeStruct, 1);

    n->label = 'E';
    n->left = n->right = NULL;

    return (SemiringItem) n;
}


static SemiringItem star(SemiringItem a)
{
    Node n = (Node) a;

    if (n->label == 'O' || n->label == 'E')
    {
        return one();
    }
    else
    {
        Node n = g_new0(struct NodeStruct, 1);

        n->label = '*';
        n->left = copy((Node) a);
        n->right = NULL;

        return (SemiringItem) n;
    }
}


static SemiringItem add(SemiringItem a, SemiringItem b)
{
    Node _a = (Node) a;
    Node _b = (Node) b;

    if (_a->label == 'O')
    {
        return copy(b);     /* O is neutral element */
    }
    else if (_b->label == 'O')
    {
        return copy(a);     /* O is neutral element */
    }
    else
    {
        Node n = g_new0(struct NodeStruct, 1);

        n->label = '+';
        n->left = copy((Node) a);
        n->right = copy((Node) b);

        return (SemiringItem) n;
    }
}


static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    Node _a = (Node) a;
    Node _b = (Node) b;

    if (_a->label == 'O' || _b->label == 'O')
    {
        return zero();       /* O is absorbing */
    }
    else if (_a->label == 'E')
    {
        return copy(b);     /* E is neutral element */
    }
    else if (_b->label == 'E')
    {
        return copy(a);     /* E is neutral element */
    }
    else
    {
        Node n = g_new0(struct NodeStruct, 1);

        n->label = '.';
        n->left = copy((Node) a);
        n->right = copy((Node) b);

        return (SemiringItem) n;
    }
}



static gboolean equals(SemiringItem a, SemiringItem b)
{
    Node _a = (Node) a;
    Node _b = (Node) b;

    if (_a == NULL || _b == NULL)
    {
        return (_a == NULL && _b == NULL);
    }
    else if (_a->label != _b->label)
    {
         return FALSE;
    }
    else
    {
        return equals((SemiringItem) _a->left, (SemiringItem) _b->left) &&
               equals((SemiringItem) _a->right, (SemiringItem) _b->right);
    }
}





static gboolean _parse(const gchar *start, const gchar **end, Node *result)
{
    const gchar *p = start;
    Node si = one();


    while (*p != 0)
    {
        if (('a' <= *p && *p <= 'z') || *p == 'O' || *p == 'E')
        {
            Node n = g_new0(struct NodeStruct, 1);
            Node product;

            n->label = *p;
            n->left = n->right = NULL;

            if (p[1] == '*')
            {
                Node s = (Node) star((Semiring) n);
                release(n);
                n = s;
                p++;
            }

            product = mult(si, n);
            release(si);
            release(n);
            si = product;

            p++;
        }
        else if (*p == '+')
        {
            Node n;
            Node sum;

            p++;
            if (!_parse(p, &p, &n))
            {
                release(n);
                *end = p;
                *result = si;
                return FALSE;
            }

            sum = add(si, n);
            release(si);
            release(n);
            si = sum;
        }
        else if (*p == '(')
        {
            Node n;
            Node product;

            p++;
            if (!_parse(p, &p, &n))
            {
                release(n);
                *end = p;
                *result = si;
                return FALSE;
            }

            product = mult(si, n);
            release(si);
            release(n);
            si = product;
        }
        else if (*p == ')')
        {
            if (p[1] == '*')
            {
                Node s;

                s = star(si);
                release(si);
                si = s;
            }

            *end = p + 2;
            *result = si;

            return (p - start > 0);
        }
        else
        {
            *end = p;
            *result = si;

            return FALSE;
        }
    }

    *end = p;
    *result = si;
    return (p - start > 0);
}



static SemiringItem from_string(const gchar *string, guint *read)
{
    const gchar *end;
    Node n;


    if (!_parse(string, &end, &n) || (end - string) < strlen(string))
    {
        release((SemiringItem) n);
        n = zero();
    }
    if (read != NULL) *read = end - string;

    return (SemiringItem) n;
}


static char* to_string(SemiringItem item)
{
    Node n = (Node) item;

    if (n == NULL)
    {
        return NULL;
    }
    else if (g_ascii_isalpha(n->label))     /* a-z, O, E */
    {
        gchar label[2];
        label[0] = n->label;
        label[1] = 0;

        return g_strdup(label);
    }
    else
    {
        gchar *left = to_string((SemiringItem) n->left);
        gchar *right = to_string((SemiringItem) n->right);
        gchar *result;

        if (n->label == '*')
        {
            if (strlen(left) == 1)
            {
                result = g_strconcat(left, "*", NULL);
            }
            else
            {
                result = g_strconcat("(", left, ")*", NULL);
            }
        }
        else if (n->label == '+')
        {
            result = g_strconcat(left, "+", right, NULL);
        }
        else
        {
            if (n->left->label == '+')
            {
                gchar *h = g_strconcat("(", left, ")", NULL);
                g_free(left);
                left = h;
            }
            if (n->right->label == '+')
            {
                gchar *h = g_strconcat("(", right, ")", NULL);
                g_free(right);
                right = h;
            }
            result = g_strconcat(left, right, NULL);
        }

        g_free(left);
        g_free(right);

        return result;
    }
}




G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
