/*
 * This file is part of AppC.
 * Copyright (C) 2004 Sascha Boehme
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "AppC.h"
#include "Fraction.h"




typedef struct TropicalStruct
{
    gboolean infinity;
    Fraction value;
} *Tropical;





static SemiringItem copy(SemiringItem item)
{
    Tropical a = (Tropical) item;
    Tropical t = g_new0(struct TropicalStruct, 1);

    t->infinity = a->infinity;
    t->value = fraction_copy(a->value);

    return (SemiringItem) t;
}



static void release(SemiringItem item)
{
    Tropical a = (Tropical) item;

    fraction_free(a->value);
    g_free(a);
}


static SemiringItem zero(void)
{
    Tropical z = g_new0(struct TropicalStruct, 1);

    z->infinity = TRUE;
    z->value = fraction_zero();

    return (SemiringItem) z;
}


static SemiringItem one(void)
{
    Tropical o = g_new0(struct TropicalStruct, 1);

    o->infinity = FALSE;
    o->value = fraction_zero();

    return (SemiringItem) o;
}


static SemiringItem star(SemiringItem a)
{
    return one();
}


static SemiringItem add(SemiringItem a, SemiringItem b)
{
    Tropical _a = (Tropical) a;
    Tropical _b = (Tropical) b;

    if (_a->infinity == TRUE)
    {
        return copy(b);
    }
    else if (_b->infinity == TRUE)
    {
        return copy(a);
    }
    else
    {
        Tropical t = g_new0(struct TropicalStruct, 1);

        t->infinity = FALSE;
        t->value = fraction_min(_a->value, _b->value);

        return t;
    }
}


static SemiringItem mult(SemiringItem a, SemiringItem b)
{
    Tropical _a = (Tropical) a;
    Tropical _b = (Tropical) b;

    if (_a->infinity == TRUE || _b->infinity == TRUE)
    {
        return zero();
    }
    else
    {
        Tropical t = g_new0(struct TropicalStruct, 1);

        t->infinity = FALSE;
        t->value = fraction_add(_a->value, _b->value);

        return t;
    }
}



static gboolean equals(SemiringItem a, SemiringItem b)
{
    Tropical _a = (Tropical) a;
    Tropical _b = (Tropical) b;

    if (_a->infinity != _b->infinity)
    {
        return FALSE;
    }
    else if (_a->infinity == TRUE)  /* && _b->infinity == TRUE */
    {
        return TRUE;
    }
    else
    {
        return fraction_equals(_a->value, _b->value);
    }
}


static SemiringItem from_string(const gchar *string, guint *read)
{
    if (!strcmp(string, "oo"))
    {
        if (read != NULL) *read = 2;
        return zero();
    }
    else
    {
        Tropical t = g_new0(struct TropicalStruct, 1);

        t->infinity = FALSE;
        t->value = fraction_from_string(string, read);

        /* accept only non-negative numbers */
        if (t->value->sign == Minus)
        {
            release((SemiringItem) t);

            if (read != NULL) *read = 0;
            return zero();
        }
        else
        {
            return t;
        }
    }
}


static char* to_string(SemiringItem item)
{
    Tropical t = (Tropical) item;

    if (t->infinity == TRUE)
    {
        return g_strdup("oo");
    }
    else
    {
        return fraction_to_string(t->value);
    }
}




G_MODULE_EXPORT void semiring_initialize(Semiring sr)
{
    sr->zero = zero;
    sr->one = one;
    sr->star = star;
    sr->add = add;
    sr->mult= mult;

    sr->equals = equals;
    sr->copy = copy;

    sr->from_string = from_string;
    sr->to_string = to_string;

    sr->release = release;
}
